function [FDist, FInd] = compute_GMM_neg_loglike(feat, means, priors, sqdetcovs, invcovs) ;
%
%   Compute negative log-likelihood
%   For each SP, find the closest component and then compute the min.
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   feat        : feature of each SP ([# of SP x dim feat])
%                 We assume that all pixels in each SP have the same feature value.
%   means       : means of GMM components
%   pirors      : priors of GMM components
%   sqdetcovs   : square of the determinant of covariance of GMM components
%   invcovs     : inverse of covariance of GMM components
%
% --------
% OUTPUTS:
%
%   FDist       : negative log-likelihood of the feature of each SP
%                 to components ([# of SP x # of components).
%   FInd        : the closest components
%
% ---------------
% Gunhee Kim
%

    MAX_VAL = 1000 ;

    num_feat = size(feat,1) ;
    Ncomp = size(means,2) ;

    FDist = zeros(num_feat, Ncomp) ;
    for c=1:Ncomp,
        % (data - mean)
        V = feat - repmat(means(:,c)', [num_feat 1]) ;

        % See Eq.(9) of the 'GrabCut 2004 paper'
        FDist(:,c) = -log( ( priors(c)/sqdetcovs(c) ) * ...
            exp( -( sum( ( (V * invcovs(:,:,c)) .* V ), 2) /2) ) );
    end

    %[FDist FInd] = min(FDist,[],2) ;
    % to avoid INF
    FDist(FDist>MAX_VAL) = MAX_VAL ;

    if nargout>1,
        [FDist FInd] = min(FDist,[],2) ;
    end

    % neg log likelihood = smaller is better.
    % So make it larger is better.
    % FDist = -FDist ;
end
