
% %%%% Global configuration file %%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% DIRECTORIES & STRINGS
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TYPE_IMG		= '.jpg' ;
TYPE_MAT        = '.mat' ;
TYPE_GT         = '.png' ;

% directory of library
DIR_LIB         = 'lib' ;
% directory of subopt_anidiff package
DIR_SUBOPT      = '../subopt_anidiff' ;

% 1. Flickr dataset
NAME_DATASET        = 'FlickrMFCdataset' ;
DIR_WORK_HOME       = ['/sdb1/data/FlickrMFCdataset/'] ;
DIR_WORK_TRAIN      = fullfile(DIR_WORK_HOME, 'train') ;
DIR_WORK_IMG        = fullfile(DIR_WORK_HOME, 'img') ;
DIR_WORK_GT         = fullfile(DIR_WORK_HOME, 'gt') ;

CLASSES = { ...
'apple+picking', ...
'baseball+kids', ...
'butterfly+blossom', ...
'cheetah+safari', ...
'cow+pasture', ...
'dog+park', ...
'dolphin+aquarium', ...
'fishing+alaska', ...
'gorilla+zoo', ...
'liberty+statue', ...
'parrot+zoo', ...
'stonehenge', ...
'swan+zoo', ...
'thinker+Rodin', ...
} ;
% CLASSES = {CLASSES{1}} ;
NCLASS = length(CLASSES) ;


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%% Add Paths
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Add path of subopt_anidiff
if ~exist('opts_single_seg'),
    addpath(DIR_SUBOPT) ;
    addpath(fullfile(DIR_SUBOPT, 'segment')) ;
    addpath(fullfile(DIR_SUBOPT, 'div_rank')) ;
    addpath(fullfile(DIR_SUBOPT, 'draw')) ;
end
% When you use normalized cuts for oversgementation algorithm
lib_path = fullfile(DIR_LIB, 'ncut_multiscale_1_6') ;
if exist(lib_path, 'dir') && ~exist('ncut_multiscale_1_6'),
    addpath('../subopt_anidiff/lib/ncut_multiscale_1_6') ;
end

% add TurboPixel path if it exists
lib_path = fullfile(DIR_SUBOPT, 'lib/TurboPixels/') ;
if exist(lib_path, 'dir'),
    if ~exist('get_speed_based_on_gradient'),   addpath(lib_path) ; end
    if ~exist('computeExtensionFields2d'),      addpath([lib_path 'lsmlib']) ; end
end
% add vlfeat path
lib_path = fullfile(DIR_SUBOPT, 'lib/vlfeat-0.9.14/') ;
if exist(lib_path, 'dir'),
    if ~exist('vl_setup'), run([lib_path 'toolbox/vl_setup']) ; end
    if ~exist('run_bilateralFilt'), addpath(fullfile(DIR_SUBOPT, 'lib/bilateralFilt')) ; end
end


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% INTERMEDIATE FILE NAME LISTS
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% file info
FOLDR_D_INFO            = 'd_info' ;
FNAME_D_INFO            = 'file_d_info.mat' ;

% save output of superpixels.
FOLDR_SUPERPIXELS       = 'SP' ;
FNAME_SUPERPIXELS       = 'fsp.mat' ;

% save output of oversegments
FOLDR_SEGM              = 'segm' ;
FNAME_SEGM              = 'fsegm.mat' ;

% VW dictionary (by script_extract_desnseVM.m)
FNAME_DIC_VLFEAT        = 'file_dic_vlfeat.mat' ;

% descriptor folder
FOLDR_DESC              = 'desc' ;
% vlfeat sift features
FOLDR_FEAT_VLSIFT   	= 'vlsift' ;

% supervised vs unsupervised
FOLDR_SUPER             = 'supervised' ;
FOLDR_UNSUP             = 'unsupervised' ;

% folder_results
FOLDR_RES               = 'result' ;
FOLDR_QUANT_RES         = 'quant_result' ;
FOLDR_RES_IMG           = 'image_result' ;

% Image group information
FNAME_GROUP_INFO        = 'GroupInfo.mat' ;


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%% Parameters
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 1. Oversegmentation option
% see the details at 'lib/subopt_anidiff/opts_single_seg.m'.
% 1: ncuts, 2: submodular segmentation
OPT_SEG.TYPE                = 2 ;
% number of oversegments
OPT_SEG.K                   = 18 ;
% maximum image dimension
OPT_SEG.MAX_IMG_SIZE        = 420 ;
% number of superpixels
OPT_SEG.NUM_SP              = 1000 ;
% margin for bounding boxes
OPT_SEG.BB_MARGIN           = 5 ;
% SIFT feature type = {'GRAY', 'HSV'}
OPT_SEG.SIFT_COLOR          = 'HSV' ;


% 2. MFG option
% 1: supervised, 0: unsupervised
OPT.SUPERVISED = 1 ;
% how much portion of images are used for labeling.
OPT.GT_RATIO = 0.2 ;
% Foreground model (0/1: On/Off, [GMM SPM]) ;
OPT.FG_MOD = [1 1] ;
% maximum number of images to be used for the initilization using diversity ranking.
OPT.UNSUP_NIMG = 200 ;
% unsupervised K (If you set OPT.SUPERVISED=1, this is ignored).
OPT.UNSUP_K = 3:12 ;
% color or grayscale SIFT ?
if strcmp (OPT_SEG.SIFT_COLOR, 'HSV'),
    OPT.BSIFTCOLOR = 1 ;
else, OPT.BSIFTCOLOR = 0 ; end

% 3. FG model option
% number of foregrounds (If you set OPT.SUPERVISED=1, this is ignored)
OPT.ForeK = 2 ;
% foreground GMM component numbers
OPT.FG_Ncomp = 5 ;
% maximum number of bounding boxes per image for background
OPT.BG_BB_NUM_PIM = 50 ;
% weight of GMM Foreground model (this value should be [0, 1]).
OPT.WEIGHT_FG_GMM = 0.95 ;

% SVM type 1: default, 2: libsvm
OPT.SVM_TYPE = 1 ;

%{
% maximum number of images to be used for the initilization using diversity ranking.
OPT.UNSUP_NBBPIMG = 5 ;
% boundbin boxes are enlarged by 5
OPT.BB_MARGIN = 5 ;
%}

%{
% Beam search ratio
OPT.MAX_BEAM_RATIO = 3 ;
% maximum iteration
OPT.MAX_ITER = 5 ;
%}



