function [trainID, OPT] = divide_train_test(bMatImFG, OPT)
%
%   divide the images into training and test sets.
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   bMatImFG    : indicates whether an image includes FG ([# FG x # img] )
%   OPT         : parameters
%
% --------
% OUTPUTS:
%
%   trainID     : indices of training images
%   OPT         : parameters
%
% ---------------
% Gunhee Kim
%

[nFG nD] = size(bMatImFG) ;

% number of images per FG set
nImgPset = full(sum(bMatImFG, 2)) ;

% image indices of each FG
setFGInd = cell(1, nFG) ;
for i=1:nFG,
    setFGInd{i} = find(bMatImFG(i,:)) ;
end

% consider the set with less images.
[~, priorityImInd] = sort(nImgPset) ;

% number of training images
nTrain = round(OPT.GT_RATIO*nD) ;

i = 0 ;
trainID = [] ;
% which FGes are chosen.
bChosenFG = zeros(1, nFG) ;
while 1

    % first consider the FG with less images.
    ii = mod(i,nFG) + 1 ;
    iFgId = priorityImInd(ii) ;
    % randomly chosen image in the set
    randInd = randperm(nImgPset(iFgId)) ;
    selImg = setFGInd{iFgId}(randInd(1)) ;
    % add it to the gt set
    trainID = [trainID selImg] ;
    % mark which FGes are chosen.
    bChosenFG(bMatImFG(:,selImg)==1) = 1 ;
    i = i + 1 ;

    % the number of chosen images as training should be larger than nTrain
    % (AND) at least one image per FG should be chosen.
    if length(trainID)>=nTrain && isempty(find(bChosenFG==0)),
        break ;
    end
end

% update the number of foregrounds
OPT.ForeK = nFG ;

end
