function [sptHist] = feat_sift_SP(imgSP, VW, VWparam)
%
%   Extract SIFT histogram from each SP
%
% -------
% INPUTS:
%
%   ImgSeg          : oversegment image
%   VW              : visual word image
%   VWparam         : VW info
%
% --------
% OUTPUTS:
%
%   sptHist         : spatial SIFT histogram
%
% ---------------
% Gunhee Kim
%

% make the size the same
spImg = fn_eql_imsize(imgSP.SPImg, VW, 1) ;

% # of SPs
nSP = length(imgSP.SPMass) ;

vw2sp = [double(VW(:)) spImg(:)] ;
vw2sp(vw2sp(:,1)==0,:) = [] ;

sptHist = sparse(vw2sp(:,1), vw2sp(:,2), ones(size(vw2sp,1),1), ...
    VWparam.NumVisualWords, nSP) ;

end
