function [sptHist] = feat_sift_segm(segImg, VW, VWparam)
%
%   Extract SIFT feature from each segment
%
% -------
% INPUTS:
%
%   ImgSeg          : oversegment image
%   VW              : visual word image
%   VWparam         : VW info
%
% --------
% OUTPUTS:
%
%   sptHist         : spatial SIFT histogram
%
% ---------------
% Gunhee Kim
%

% # of segments
nSeg = max(unique(segImg(:))) ;

% make the size the same
segImg = fn_eql_imsize(segImg, VW, 1) ;

% SIFT histogram
sptHist = zeros(VWparam.NumVisualWords, nSeg) ;
histInd = 0:VWparam.NumVisualWords ;
for i=1:nSeg,
    ihist = histc(VW(segImg==i), histInd) ;
    sptHist(:,i) = ihist(2:end) ;
end

sptHist = single(sptHist) ;

end
