function [D, Folder, nImFolder] = fn_get_fildDB(varargin)
%
%	Get file DB given parent folder and extensions.
%   This file is based on LMdatabase.m of LabelMe Toolbox.
%
% -------
% INPUTS:
%
%   varargin{1} :       HOMEIAMGES (parent folder path)
%   varargin{2} :       IMG_EXT (file extensions to be searched)
%
% --------
% OUTPUTS:
%
%   D:          file DB. length(D) = number of files.
%               Two fields: D(i).folder: folder name, D(i).fname: file name
%   Folders:    list of all searched folders.
%   nImFolder:  number of files of each searched folder.
%
% -------------
%
% Gunhee Kim
%

Folder = [];

% Parse input arguments and read list of folders
N = nargin;

% get image folder
if N<1,
    HOMEIMAGES = '';
else,
    HOMEIMAGES = varargin{1};
    if HOMEIMAGES(end) == '/' || HOMEIMAGES(end) == '\',
        HOMEIMAGES = HOMEIMAGES(1:end-1);
    end
end

% get image extension
if N<2, 		IMG_EXT = '.jpg' ;
else,           IMG_EXT = varargin{2};
end

% if multiple image extensions are given,
if iscell(varargin{N})
    folderlist = varargin{N}
    Nfolders = length(folderlist);
    for i = 1:Nfolders
        Folder{i} = folderlist{i};
    end
% otherwise,
else
    % get all folders
    folders = genpath(HOMEIMAGES);
    % parse the folders into FOlder
    h = [findstr(folders,  pathsep)];
    h = [0 h];
    Nfolders = length(h)-1 ;
    for i = 1:Nfolders
        tmp = folders(h(i)+1:h(i+1)-1);
        tmp = strrep(tmp, HOMEIMAGES, ''); tmp = tmp(2:end);
        Folder{i} = tmp;
    end
end

% Loop on folders
D = []; n = 0;
nImFolder = zeros(Nfolders,1) ;
for f = 1:Nfolders,

    folder = Folder{f};

    % get iamges under the folder.
    filesImages = dir(fullfile(HOMEIMAGES, folder, ['*' IMG_EXT]));
    N = length(filesImages);

    % get file DB
    for i = 1:N
        n = n+1 ;
        filename = fullfile(HOMEIMAGES, folder, filesImages(i).name);
        D(n).folder = folder;
        D(n).fname = filesImages(i).name;
    end


    nImFolder(f) = N ;
    %disp(sprintf('%d/%d, %s, %d images added, Total: %d', f, Nfolders, ...
    %             folder, N, n)) ;

end
