function [mst] = fn_kruskal (A)
%
%   generate minimum spanning tree using kruskal's method. 
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   A       : input network
%
% --------
% OUTPUTS:
%
%   mst     : minimum spanning tree
%
% ---------------
% Gunhee Kim
%

%
%---------------------kruskal.m---------------------   
%
% Kruskal algorithm for finding minimum spanning tree
%
% Input:  PV = nx3 martix. 1st and 2nd row's define the edge (2
% vertices) and
% the 3rd is the edge's weight
% Output: w = Minimum spanning tree's weight
%         T = Minimum spanning tree's adjacency matrix
% 
% N.Cheilakos,2006
%----------------------------------------------------


% if the matrix is symmetric,
if isempty(find((A-A')~=0)), 
    A = tril(A,-1) ;
% if not,
else
    A = A + A' ;
    A = tril(A,-1) ;
end

nA = size(A,1) ;
[rA cA vA] = find(A) ;
PV = [rA cA vA] ; 
clear rA cA vA ;

nnzE = size(PV,1) ;

% sort edges in an ascending order of weights
[sort_vA sort_ind] = sort(PV(:,3)) ;
PV = PV(sort_ind,:) ;


korif = zeros(1,nA);
for i = 1 : nnzE
    % cycle detection
    [korif,c] = iscycle(korif, PV(i,[1 2]));
    if c == 1
       PV(i,:) = [0 0 0];
   end
end

% remove bad edges
PV(sum(PV,2)==0,:) = [] ;


% final MST
mst = sparse(PV(:,1), PV(:,2), PV(:,3), nA, nA) ;


end

%
%--------------------iscycle.m---------------------------
%
% input: korif = set of vertices in the graph
%       akmi = edge we insert in graph
% output: korif = The "new: set of vertices
%        c = 1 if we have circle, else c = 0
%
% N.Cheilakos,2006
%---------------------------------------------------------
function [korif,c] = iscycle(korif,akmi)

g=max(korif)+1;
c=0;
n=length(korif);
if korif(akmi(1))==0 & korif(akmi(2))==0
    korif(akmi(1))=g;
    korif(akmi(2))=g;
elseif korif(akmi(1))==0
    korif(akmi(1))=korif(akmi(2));
elseif korif(akmi(2))==0
    korif(akmi(2))=korif(akmi(1));
elseif korif(akmi(1))==korif(akmi(2))
    c=1;
else
    m=max(korif(akmi(1)),korif(akmi(2)));
    for i=1:n
        if korif(i)==m
           korif(i)=min(korif(akmi(1)),korif(akmi(2)));
       end
   end
end

end