function [data] = fn_norm_neg_loglike(data) ;
%
%   Normalize the negative log likelihood into [0,1]
%   Note that the smaller is better in the negative log likelihood.
%   We set the minimum to 1 and the maximum to 0, and then
%   interpolate linearly.
%   Finally, 1 is the best and 0 is the worst.
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   data        : input data
%
% --------
% OUTPUTS:
%
%   data        : output
%
% ---------------
% Gunhee Kim
%

% maximum value
MAX_VAL = max(data(:)) ;

% min and max
min_data = min(data(:)) ;
% usually, max val is thresholded by a really big number
max_data = max(data(data<MAX_VAL)) ;


% slope (min -> 1, max -> 0)
slope = 1/(min_data - max_data) ;

data = slope * (data - max_data) ;

data(data<0) = 0 ;

end
