function [valSPM] = fn_runsvm_segm (FG_SPM, sptHist, mergedPair, SVM_TYPE)
%
%   run svm for the BB
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   FG_SPM :        svm model
%   sptHist :       feature of BaseBB ([dimFeat x nBaseBB])
%   mergedPair :    pairs to be merged (# of pairs x 2])
%   SVM_TYPE :      SVM type
%
% --------
% OUTPUTS:
%
%   valSPM :        svm scores ([nBB x 1])
%
% ---------------
% Gunhee Kim
%

if nargin < 4,
    SVM_TYPE = 1 ;
end

dimHist = size(sptHist,1) ;
nPair = size(mergedPair,1) ;

% get pair hist
pairHist = zeros(dimHist, nPair);
for i=1:size(mergedPair,2),
    pairHist = pairHist + sptHist(:,mergedPair(:,i)) ;
end
% L1 normalization
pairHist = pairHist ./ repmat(sum(pairHist), [dimHist 1]) ;

if SVM_TYPE == 1,

    % get kernel (using histogram intersection)
    K_test = vl_alldist2(FG_SPM.SVdata, double(pairHist), 'kl1');

    % See 'svm_one_vs_all.m' and 'kernel.m'
    % in 'sunfeature/SUN_source_code_v2/code/scene_sun/'.
    % score_test = FG_SPM.sv_coef' * K_test(FG_SPM.SVs,:) - FG_SPM.rho ;
    score_test = FG_SPM.sv_coef' * K_test - FG_SPM.rho ;

    valSPM = score_test(:) ;
else,
    % run svm
    [~, ~, prob_est] = svmpredict( ones(nPair,1), ...
        double(pairHist)', FG_SPM, '-b 1');

    % final score
    valSPM = prob_est(:,1) ;
end
