function [wcov] = fn_wcov (data, wgt)
%
%   weigted covariance matrix
%   See http://en.wikipedia.org/wiki/Covariance
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   data        : input data ([nData x nDim])
%   wgt         : weights ([nData x 1])
%
% --------
% OUTPUTS:
%
%   wcov        : weighted cov ([nDim x nDim])
%
% ---------------
% Gunhee Kim
%

% if no weight, use ordinary 'cov' function
if nargin<2,
    wcov = cov(data) ;
else

    % compute weighted mean
    mean_data = fn_wmean(data, wgt) ;
    % mean data
    mean_data = repmat(mean_data, [size(data,1) 1]) ;

    wcov = (data - mean_data) .* repmat(sqrt(wgt), [1 size(data,2)]) ;
    wcov = wcov' * wcov ;

    wcov = wcov / ( sum(wgt) - 1) ;

end

end
