function [cand_mst cand_graph] = get_GMM_MST_FG(segm_adjmat, ...
                DistSeg2FG, massSeg, flimg, fsegm)
%
%   get candidate graph and candidate tree (mst) for GMM foreground model
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   segm_adjamt     : adjacenty graph of segments
%   DistSeg2FG      : distance of segments to the given foreground
%   massSeg         : pixel numbers of segments
%   flimg           : the original image
%   fsegm           : segmentation info
%
% --------
% OUTPUTS:
%
%   cand_mst        : candidate tree (MST) (Lower Triangular Matrix)
%   cand_graph      : candidate graph (Lower Triangular Matrix)
%
% ---------------
% Gunhee Kim
%

% number of segments
num_seg = length(DistSeg2FG) ;

if nargin < 3,
    massSeg = ones(num_seg, 1) ;
end

% if nargin < 4,  do_draw = 0 ;
% else            do_draw = 1 ;
% end

% b draw
do_draw = 0 ;


% % --------------------------------------------------------------%
%       1. Construct weighted connectivity graph
% % --------------------------------------------------------------%

%{
% (1) Method 1: just sum of log-likelihood

% (a) diagonal elements ( (i,i): negative log-likelihood i-th segment )
cand_graph = diag(DistSeg2FG) ;
% (b) off-diagonal elements
% Get connectivity graph
[rr cc] = find(segm_adjmat~=0) ;
[rc] = find(segm_adjmat~=0) ;
cand_graph(rc) = DistSeg2FG(rr) + DistSeg2FG(cc) ;
%}

% (2) Method 2: average of log-likelihood ;
% (a) diagonal elements
% (i,i): negative log-likelihood i-th segment)
cand_graph = diag(DistSeg2FG ./ massSeg) ;
% (b): off-diagonal elements
% (i,j): mean of negative log-likelihood of i-th and j-th segments
[rr cc] = find(segm_adjmat ~= 0) ;
[rc] = find(segm_adjmat ~= 0) ;
cand_graph(rc) = (DistSeg2FG(rr) + DistSeg2FG(cc)) ./ (massSeg(rr) + massSeg(cc)) ;
cand_graph(isnan(cand_graph)) = 0 ;


% % --------------------------------------------------------------%
%       2. Run Kruskal algorithm
% % --------------------------------------------------------------%

% Build maximum spanning tree
cand_mst = fn_kruskal(-cand_graph) ;

% get the correct values
cand_mst(cand_mst~=0) = cand_graph(cand_mst~=0) ;

% make the graph sparse
cand_graph = sparse(cand_graph) ;

% (DEBUG) display
if do_draw == 0, return ; end


% % --------------------------------------------------------------%
%       3. (DEBUG) Display
% % --------------------------------------------------------------%

end


