function [simSeg2Comp massSeg] = get_GMM_simSeg2Comp(imgSP, descSP, FG_GMM, OPT)
%
%   get similarity between segments and components of Foregrounds
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   imgSP :     information about superpixels
%   descSP :    descriptors of superpixels
%   FG_GMM :    GMM Foreground model
%   OPT :       parameters
%
% --------
% OUTPUTS:
%
%   simSeg2Comp :   similarity between segments and components of
%                   foregrounds ([num_seg x num_comp])
%   massSeg :       mass (or weights) of Segs ([num_seg x num_comp])
%
% ---------------
% Gunhee Kim
%

% number of segments and SPs
num_seg = length(imgSP.SPIndPSeg) ;
num_sp = length(imgSP.SP2Seg) ;

% compute negative log-likelihood of the feature of every superpixels
% to the components
DistSP2Comp = zeros(num_sp, OPT.ForeK+1) ;
for k=1:OPT.ForeK+1,
    [DistSP2Comp(:,k) IndSP2Comp] = compute_GMM_neg_loglike(descSP.RGB, FG_GMM(k).means, ...
        FG_GMM(k).priors, FG_GMM(k).sqdetcovs, FG_GMM(k).invcovs) ;
end

% Normalize the negative log likelihood into [0,1]
% Note that the smaller is better in the negative log likelihood.
% We set the minimum to 1 and the maximum to 0, and then interpolate linearly.
% Finally, 1 is the best and 0 is the worst.
SimSP2Comp = fn_norm_neg_loglike ( DistSP2Comp ) ;

% negative log-likelihood of every segment to the component
% # of pixels per segment
% See Eq.(8) of GrabCut 2004.
simSeg2Comp = zeros(num_seg, OPT.ForeK) ;
massSeg = zeros(num_seg, OPT.ForeK) ;
for k=1:OPT.ForeK+1,
    for v=1:num_seg,
        simSeg2Comp(v,k) = sum(SimSP2Comp(imgSP.SPIndPSeg{v},k) .* ...
            imgSP.SPMass(imgSP.SPIndPSeg{v})) ;
        massSeg(v,k) = sum(imgSP.SPMass(imgSP.SPIndPSeg{v})) ;
    end
end

end

