function [OptFGIdSeg OptFGIdSP segOutImg] = get_seg_from_opt_cand( ...
                   optSlnCand, setCandFg, imgSP)
%
%   Obtain segmentation output from optimal FG candidates
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   optSlnCand  : optimal selection of FG candidates ([B x 2])
%                 (FG_ID, Bid ID)
%   setCandFg   : candiate sets ([1 x K] cells)
%   imgSP       : information about superpixels
%
% --------
% OUTPUTS:
%
%   OptFGIdSeg  : optimal FG assigment for segments
%   OptFGIdSP   : optimal FG assignment for SPs
%   segOutImg   : final segmentation image
%
% ---------------
% Gunhee Kim
%

% number of opt bids
nOpt = size(optSlnCand,1) ;

% number of segments and SPs
nSeg = length(imgSP.SPIndPSeg) ;
numSP = length(imgSP.SP2Seg) ;


optCandMat = zeros(nOpt, nSeg) ;
for o=1:nOpt,
    optCandMat(o,:) = setCandFg{optSlnCand(o,1)}(optSlnCand(o,2),:) ;
end


% cluster ID for each segment
OptFGIdSeg = zeros(nSeg,1) ;
for o=1:nOpt,
    OptFGIdSeg(optCandMat(o,:)>0) = optSlnCand(o,1) ;
end
% cluster ID for each SP
OptFGIdSP = ones(numSP,1) ;
for o=1:nSeg,
    OptFGIdSP(imgSP.SP2Seg==o) = OptFGIdSeg(o) ;
end

% draw region assignment
segOutImg = get_segm_img_from_sp(imgSP.SPImg, OptFGIdSP) ;

end
