function SP = get_superpixel_from_image (flimg, MAX_IMG_SIZE, NUM_SP, SP_MODE)
%
%  Generate superpixel from each image
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%   flimg :         input image path
%   MAX_IMG_SIZE :  max image size to be resized
%   NUM_SP :        number of superpixels
%   SP_MODE :       Superpixel method
%
% --------
% OUTPUTS:
%   SP :            superpixel info
%       .numSP :    number of SPs.
%       .flimg :    image path
%       .numSP :    number of superpixels
%       .imgSP :    superpixel images, 
%       .adjMat :   Neighborhood matrix btw segments
%       .cntSeg :   center point of each segment. 
%       .massSP :   number of pixels per superpixel ([1 x numSP])
%
% ---------------
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

    % default parameters
    if nargin<2,    MAX_IMG_SIZE = 256 ;    end
    if nargin<3,    NUM_SP = 1000 ;         end
    if nargin<4,    SP_MODE = 3 ;           end

    % load and resize an image.
    img = imread(flimg) ;
    [nr nc nch] = size(img) ;
    imsize_org = [nr nc nch] ;
    im_res_rate = min(MAX_IMG_SIZE ./ [nr nc]) ;
    img = fn_imresize(img,im_res_rate) ;

    [spImg, AdjMat, cntSeg] = run_superpixel(img, NUM_SP, SP_MODE) ;

    % Each SP is weighted by its number of pixel
    massSP = histc(spImg(:), [0:size(cntSeg,1)]) ;
    massSP(1) = [] ;

    SP.imSize = imsize_org ;
    SP.flimg = flimg ; 
    SP.numSP = size(cntSeg,1) ;
    SP.imgSP = uint16(spImg) ;
    SP.adjMat = AdjMat ;
    SP.cntSeg = cntSeg ;
    SP.massSP = massSP ;
end
