function [FG_SPM] = run_SPM_FG_init (descFG, SVM_TYPE)
%
%   Get initial SPM foreground models, which are linear SVMs on
%   SIFT features
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   SPMdescFG   : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : SIFT feature values ([Dim x # of features])
%       .imgID  : image ID
%
% --------
% OUTPUTS:
%
%   FG_SPM      : SPM FG SVM model
%               if SVM_TYPE == 1,
%               See the detail at 'train_svm_1vsall.m'.
%               if SVM_TYPE == 2,
%               See the README of 'lib/libsvm-mat-2.89-3' for the details
%
% ---------------
% Gunhee Kim
%

if nargin < 2,
    SVM_TYPE = 1;
end

% get labels for data
foreK = length(descFG) ;
label_FG_inst = [] ;
for k=1:foreK,
    label_FG_inst = [label_FG_inst k*ones(1,size(descFG(k).desc,2)) ] ;
end
% concatenate the data
data = double(horzcat(descFG(:).desc)) ;
ndata = size(data,2) ;
%clear descFG ;

% for each foreground,
% all the other points becomes negative samples
for k=1:foreK,
    label_ind = -1*ones(ndata,1) ;
    label_ind(label_FG_inst==k) = 1 ;

    % run linear svm
    if SVM_TYPE == 1,
        % A more faster way is to use the idea of sunfeat.
        % See 'compute_kernel_svm_train.m', 'svm_one_vs_all.m', and
        % 'kernel.m' in 'sunfeature/SUN_source_code_v2/code/scene_sun/'.
        model = train_svm_1vsall(label_ind, data) ;
    else
    % -b 1 : probability_estimates
    % -q : disable screen output of svm-train
    % this idea is a bit slow. (-b 0) is much faster because it does not run cross validation.
    % http://www.csie.ntu.edu.tw/~cjlin/libsvm/faq.html#f425
        model = svmtrain(label_ind, data', '-q -b 1');
    end

    FG_SPM(k) = model ;
end


end
