function script_extract_feat (CLASS_ID_S, CLASS_ID_T)
%
%   Extract features from oversegments
%   It will extract color/texture features from each superpixel,
%   and SIFT features from each oversegment.
%   (Note that there are several other parameters at the top of the code)
%
% -------
% INPUTS:   NONE.
%
% --------
% OUTPUTS:  NONE.
%
% --------
% DEPENDENCY:
%
% ---------------
% Written by Gunhee Kim @ CMU, 2012.
% All rights reserved.
%

% run config file
conf_file = 'config_flickr' ;
eval(conf_file);

% % ------------------------------------------------------------------------%
% %     Add path
% % ------------------------------------------------------------------------%

% If you use condor
if nargin>0,
    CL_IND = (str2num(CLASS_ID_S)+1):(str2num(CLASS_ID_T)+1) ;
else
    CL_IND = 1:NCLASS ;
end

% % ------------------------------------------------------------------------%
% %     Main function
% % ------------------------------------------------------------------------%

% for each object,
for h=1:length(CL_IND),
    extract_feat(conf_file, CLASSES{CL_IND(h)}) ;
end

% exit

end


% extract SIFT+visual words using LabelMe toolbox
function extract_feat(conf_file, obj_name)

% run config file
eval(conf_file);

% load sift feature info
fsift = fullfile(DIR_WORK_TRAIN, FNAME_DIC_VLFEAT) ;
if strcmp(OPT_SEG.SIFT_COLOR, 'HSV')
    fsift = strrep(fsift, '.mat', 'HSV.mat') ;
end
if exist(fsift),
    load(fsift, 'VWparam') ;
else,
    disp('First run [script_gen_vlvwdic.m].') ;
end
% visual dictionary
visualwordcenters = single(VWparam.visualwordcenters') ;
% sift dimension
dim_sift = sum(VWparam.NumVisualWords*(4.^(0:VWparam.Mw-1))) ;

% load file info
DIR_D_INFO = fullfile(DIR_WORK_TRAIN, FOLDR_D_INFO, obj_name) ;
DIn = fn_get_fildDB(DIR_D_INFO, FNAME_D_INFO) ;
DAll = [] ;
for i=1:length(DIn),
    load(fullfile(DIR_D_INFO, DIn(i).folder, DIn(i).fname), 'D') ;
    DAll = [DAll D] ;
end
D = DAll ; DAll = [] ;
nD = length(D) ;

% make folder to save SP
DIR_IMG = fullfile(DIR_WORK_IMG, obj_name) ;
DIR_WORK_SP = fullfile(DIR_WORK_TRAIN, ...
    [FOLDR_SUPERPIXELS num2str(OPT_SEG.NUM_SP)], obj_name) ;
DIR_WORK_SEGM = fullfile(DIR_WORK_TRAIN, [FOLDR_SEGM ...
    num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)], obj_name) ;
DIR_WORK_DESC = fullfile(DIR_WORK_TRAIN, [FOLDR_DESC ...
    num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)], obj_name) ;
if ~exist(DIR_WORK_DESC,'dir'),	mkdir(DIR_WORK_DESC) ;	end

% for each image
for i=1:nD,
    % load descriptors
    fdic = fullfile(DIR_WORK_DESC, D(i).folder) ;
    if ~exist(fdic, 'dir'), mkdir(fdic) ; end
    ffeat = fullfile(fdic, [D(i).fname(1:end-4), TYPE_MAT]);

    % if already done, skip it.
    % if exist(ffeat), continue; disp('skip') ; continue; end

    % load and resize an image
    flimg = fullfile(DIR_IMG, D(i).folder, D(i).fname) ;

    %try
    % load superpixels and segments
    fsegm = fullfile(DIR_WORK_SEGM, D(i).folder, strrep(D(i).fname, TYPE_IMG, TYPE_MAT));
    load (fsegm, 'imgSP', 'imgSeg') ;

    % extract possible bounding boxes
    % [BBAll BBsegm] = extract_boundbox_seg(imgSeg, OPT_SEG.BB_MARGIN) ;

    % load image
    img = imread(flimg) ;
    % color image only !!
    % if size(img,3)~=3, disp('Color images only!!') ; continue ; end

    % Extract color features from each SP after resizing image accordingly.
    % descSpCLR: RGB, HSV, or CIELab descriptor of super-pixel
    descSP = feat_sp_clrtxt(fn_eql_imsize(img, imgSeg.segImg), double(imgSP.SPImg));

    % Extract gray/color SIFT features from each segment
    sptHist = [] ; sptHistHSV = [] ;

    % file name
    fdesc = strrep(flimg, DIR_WORK_IMG, fullfile(DIR_WORK_TRAIN, FOLDR_FEAT_VLSIFT));
    fdesc = strrep(fdesc, TYPE_IMG,'.mat') ;
    if strcmp(OPT_SEG.SIFT_COLOR, 'HSV'),
        fdesc = strrep(fdesc, FOLDR_FEAT_VLSIFT, [FOLDR_FEAT_VLSIFT 'HSV']) ;
    end
    % make the directory of VW if it does not exist
    ddesc = fileparts(fdesc) ;
    if ~exist(ddesc, 'dir'), mkdir(ddesc) ; end

    % Extract gray/color SIFT features from each segment
    % HSV SIFT
    if strcmp(OPT_SEG.SIFT_COLOR, 'HSV'),
        % load or generate dense SIFT feature extraction.
        if exist(fdesc, 'file'),
            load(fdesc, 'VW') ;
        else,
            VW = get_vw(img, VWparam, visualwordcenters) ;
        end
        save(fdesc, 'VW') ;
        % get SIFT histogram for each BB
        % sptHistHSV = feat_sift_segm(imgSeg.segImg, VW, VWparam) ;
        sptHistHSV = feat_sift_SP(imgSP, VW, VWparam) ;
    % Gray SIFT
    else,
        if exist(fdesc, 'file'),
            load(fdesc, 'VW') ;
        else,
            VW = get_vw(img, VWparam, visualwordcenters) ;
        end
        save(fdesc, 'VW') ;
        % get SIFT histogram for each BB
        sptHist = feat_sift_SP(imgSP, VW, VWparam) ;
    end
    clear VW ;

    % save the output
    save (ffeat, 'descSP', 'sptHist', 'sptHistHSV') ;

    % display
    if mod(i,10) == 0,
        disp([obj_name ' : ' num2str(i) '/' num2str(nD)]) ;
    end

    % handling exception
    %catch, disp(['error : ' ffeat]) ; continue; end
end

end

% get visual words image
function VW = get_vw(im, VWparam, visualwordcenters)

	imr = size(im,1); imc = size(im,2) ;

	% run dense sift
	% frames [X Y Scale Orientation (in radian)]
	% descrs (class uint8)
	if strcmp(VWparam.Ocolor,'GRAY') ;
	    [frames, descrs]=vl_phow(im2single(im), 'Step', 2, ...
		'Sizes', VWparam.dense_size) ;
	else
	    [frames, descrs]=vl_phow(im2single(im), 'Step', 2, ...
		'Sizes', VWparam.dense_size, 'Color', VWparam.Ocolor) ;
	end

	% vector quantization
	[fitd, w] = min(distMat(visualwordcenters, single(descrs'))) ;

	%{
	% save [r c w]
	VW.r = uint16(frames(2,:)) ;
	VW.c = uint16(frames(1,:)) ;
	VW.w = uint8(w) ;
	%}

	% make a uint8  image
	VW_i = sparse(frames(2,:), frames(1,:), w, imr, imc) ;
	VW = uint16(full(VW_i)) ;
end

% Euclidian distances between vectors
function D=distMat(P1, P2)

if nargin == 2
    X1=repmat(single(sum(P1.^2,2)),[1 size(P2,1)]);
    X2=repmat(single(sum(P2.^2,2)),[1 size(P1,1)]);
    R=P1*P2';
    D=X1+X2'-2*R;
else
    % each vector is one column
    X1=repmat(sum(P1.^2,1),[size(P1,2) 1]);
    R=P1'*P1;
    D=X1+X1'-2*R;
    D = sqrt(D);
end

end
