function script_gen_vl_vwdic(conf_file, strObj)
%
%	Generate SIFT VW dictionary using vlfeat toolbox.
%
% -------
% INPUTS:
%   conf_file       : configuration file name
%   strObj          : object class
%
% --------
% OUTPUTS:
%
%
% -------------
%
% Gunhee Kim

% % --------------------------------------------------------------%
%      Configuration
% % --------------------------------------------------------------%

% default: config file
if nargin<1,
    conf_file = 'config_flickr' ;
end

% run config file
eval(conf_file)
disp(conf_file)

% FOR condor
if nargin>1,
    CLASSES{1} = strObj; NCLASS = 1 ;
end

strClasses = [] ;
%{
for i=1:NCLASS,
    strClasses = [strClasses CLASSES{i} '_'] ;
end
%}

% % --------------------------------------------------------------%
%      SIFT PARAMETERS
% % --------------------------------------------------------------%

% normalized image size (images will be scaled so that the maximal axis has this dimension before computing the sift features)
VWparam.imagesize = OPT_SEG.MAX_IMG_SIZE ;
% distance between grid centers
VWparam.grid_spacing = 2 ;
% See Sizes in vl_phow.
VWparam.dense_size = 4 ;
% size of patch from which to compute SIFT descriptor (it has to be a factor of 4)
VWparam.patch_size = VWparam.dense_size*VWparam.dense_size ;
% number of visual words
VWparam.NumVisualWords = 300;
% number of spatial scales for spatial pyramid histogram
VWparam.Mw = 2;
% number of randomly selected iamges for dictionary formation
% note that feature extraction takes 0.3 sec per image. (HSV)
VWparam.NumImgDict = 10000 ;
% number of samples per image
VWparam.Nsamples = 20 ;
% color option (See http://www.vlfeat.org/mdoc/VL_PHOW.html)
% GRAY (PHOW-gray), RGB, HSV, and OPPONENT (PHOW-color).
%VWparam.Ocolor = OPT_SEG.SIFT_COLOR ;
VWparam.Ocolor = 'HSV' ; % 'GRAY' ;
% CONSTANTS (you can not change this)
w = VWparam.patch_size/2; 		% boundary

% % --------------------------------------------------------------%
%      Randomly Sample images up to VWparam.NumImgDict
% % --------------------------------------------------------------%

% number of images per class to be processed
num_img_pob = round(0:(VWparam.NumImgDict/NCLASS):VWparam.NumImgDict) ;
num_img_pob = num_img_pob(2:end) - num_img_pob(1:end-1) ;

DAll = [] ;
for h=1:NCLASS,
    OBJCLASS = CLASSES{h} ;
    % load Directory info
    %DIR_IMG = fullfile(DIR_WORK_IMG, OBJCLASS) ;
    DIR_D_INFO = fullfile(DIR_WORK_TRAIN, FOLDR_D_INFO, OBJCLASS) ;
    %DIR_OBJ_IMG = fullfile(DIR_WORK_IMG, OBJCLASS ) ;

    if ~exist(DIR_D_INFO, 'dir'),
        error('First run [script_file_gen_D_info.m] !') ;
    end
    % get file info
    [D_file, Folder, nImFolder] = fn_get_fildDB(DIR_D_INFO, FNAME_D_INFO) ;

    h_DAll = [] ;
    for sub=1:length(D_file),
        % load directory infos
        load(fullfile(DIR_D_INFO, D_file(sub).folder, D_file(sub).fname), 'D') ;
        h_DAll = [h_DAll D] ;
    end

    % if loaded images are so many, randomly choose some of them.
    if length(h_DAll) > num_img_pob,
        h_DAll = h_DAll(randperm(length(h_DAll))) ;
        h_DAll = h_DAll(1:num_img_pob) ;
    end

    for i=1:length(h_DAll),
        h_DAll(i).folder = fullfile(OBJCLASS, h_DAll(i).folder) ;
    end
    DAll = [DAll h_DAll] ;
end
clear h_DAll ;

% % --------------------------------------------------------------%
%      Extract Features for the selected images
% % --------------------------------------------------------------%

Nimages = length(DAll) ;

if strcmp(VWparam.Ocolor,'GRAY'),
    P = zeros([128 Nimages*VWparam.Nsamples], 'uint8');
else
    P = zeros([128*3 Nimages*VWparam.Nsamples], 'uint8');
end

tic
for i=1:Nimages,

    % load image
    fname = fullfile(DIR_WORK_IMG, DAll(i).folder, DAll(i).fname);
    im = imread(fname) ;
    % size the image
    im = fn_imresize(im, VWparam.imagesize/max(size(im))) ;

    % run dense sift
    % frames [X Y Scale Orientation (in radian)]
    % descrs (class uint8)
    if strcmp(VWparam.Ocolor,'GRAY') ;
        [frames, descrs]=vl_phow(im2single(im), 'Sizes', VWparam.dense_size, ...
            'Step', VWparam.grid_spacing) ;
    else
        [frames, descrs]=vl_phow(im2single(im), 'Sizes', VWparam.dense_size, ...
            'Step', VWparam.grid_spacing, 'Color', VWparam.Ocolor) ;
    end

    i_rand_ind = randperm(size(descrs,2)) ;
    i_rand_ind = i_rand_ind(1:VWparam.Nsamples) ;

    P(:,(i-1)*VWparam.Nsamples+1:i*VWparam.Nsamples) = ...
        descrs(:,i_rand_ind) ;

    if mod(i,20)==0, t2 = toc ; tic ;
        disp([num2str(i) '/' num2str(Nimages) ' in ' num2str(t2) 'sec.']) ;
    end
end
disp('sift feature extraction is done') ;

%save tmp.mat DAll P ;

% % --------------------------------------------------------------%
%      Run kmeans
% % --------------------------------------------------------------%

tic
% integer k-means
% [C,A] = vl_ikmeans(P, VWparam.NumVisualWords) ;
% single or double digit k-means
[C,A] = vl_kmeans(single(P), VWparam.NumVisualWords) ;

VWparam.visualwordcenters = C ;
t2 = toc

disp(['kmeans is done in ' num2str(t2) 'sec,']) ;

% % --------------------------------------------------------------%
%      Save the result
% % --------------------------------------------------------------%

% save the result
if strcmp(VWparam.Ocolor,'GRAY'),
    save(fullfile(DIR_WORK_TRAIN, [strClasses FNAME_DIC_VLFEAT]), 'DAll', 'VWparam') ;
else,
    save(fullfile(DIR_WORK_TRAIN, [strClasses strrep(FNAME_DIC_VLFEAT, '.mat', [VWparam.Ocolor '.mat'])]), ...
        'DAll', 'VWparam') ;
end

end

