function script_mfc_flickr ()
%
%  Run MFC demo with FlickrMFC dataset
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%
% --------
% OUTPUTS:
%
% ---------------
% Gunhee Kim
%

close all; warning off;

% run config file
% default: config file
if nargin<1,
    conf_file = 'config_flickr' ;
end
eval(conf_file);

% % --------------------------------------------------------------%
%     Add Pathes
% % --------------------------------------------------------------%

% add path for svmlib
if OPT.FG_MOD(2)>0 && ~exist('libsvmread'),
    addpath('lib/libsvm-mat-2.89-3') ;
end

% % --------------------------------------------------------------%
%     Main Part
% % --------------------------------------------------------------%

% make folder
for h=1:NCLASS,
    run_main_loop(conf_file, CLASSES{h}) ;
end

% exit ;

end

% run the tracking for each class
function run_main_loop(conf_file, obj_name)

% % --------------------------------------------------------------%
%      Parameters
% % --------------------------------------------------------------%

bDebug = 0 ;

% run config file
eval(conf_file);

% % --------------------------------------------------------------%
%     Folders paths
% % --------------------------------------------------------------%

% images, gt, SP, and segment paths
DIR_PATH_IMG = fullfile(DIR_WORK_IMG, obj_name) ;
DIR_PATH_GT = fullfile(DIR_WORK_GT, obj_name) ;
DIR_WORK_SEGM = fullfile(DIR_WORK_TRAIN, [FOLDR_SEGM ...
    num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)], obj_name) ;
DIR_WORK_DESC = fullfile(DIR_WORK_TRAIN, [FOLDR_DESC ...
    num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)], obj_name) ;

% Supervised / Unsupervised output folder
if OPT.SUPERVISED>0,
    DIR_WORK_RES = fullfile(DIR_WORK_TRAIN, FOLDR_RES, FOLDR_SUPER, ...
        [num2str(OPT.ForeK) '_' num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)]) ;
else
    DIR_WORK_RES = fullfile(DIR_WORK_TRAIN, FOLDR_RES, FOLDR_UNSUP, ...
        [num2str(OPT.ForeK) '_' num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)]) ;
end


% % --------------------------------------------------------------%
%      Load Images
% % --------------------------------------------------------------%

% load file DB
DIR_D_INFO = fullfile(DIR_WORK_TRAIN, FOLDR_D_INFO, obj_name) ;
DIn = fn_get_fildDB(DIR_D_INFO, FNAME_D_INFO) ;
DAll = [] ;
for i=1:length(DIn),
    load(fullfile(DIR_D_INFO, DIn(i).folder, DIn(i).fname), 'D') ;
    DAll = [DAll D] ;
end
D = DAll ; DAll = [] ;
nD = length(D) ;

% image paths
Lflimg = cell(1,nD) ;
for i=1:nD,
    Lflimg{i} = fullfile(DIR_PATH_IMG, D(i).folder, D(i).fname) ;
end

% SP, segment, and descriptors
Lfsegm = cell(1,nD) ;
Lffeat = cell(1,nD) ;
for i=1:nD,
    tfname = strrep(D(i).fname, TYPE_IMG, TYPE_MAT) ;
    Lfsegm{i} = fullfile(DIR_WORK_SEGM, D(i).folder, tfname) ;
    Lffeat{i} = fullfile(DIR_WORK_DESC, D(i).folder, tfname) ;
end

% ground truth paths (NOTE THAT THIS INFOR DOES NOT USE IN THE ALGORITHM)
Lflgt = cell(1,nD) ;
for i=1:nD,
    Lflgt{i} = fullfile(DIR_PATH_GT, D(i).folder, ...
        strrep(D(i).fname, TYPE_IMG, TYPE_GT) ) ;
end

clear D ;

%{
% (DEBUG) display the images of objects
canvas_img = draw_imgs_big_canvas(Lflimg) ;
figure; imshow(canvas_img) ;
keyboard ;
%}

%{
% (DEBUG) display the ground truth of objects
cmap = VOClabelcolormap(255);
canvas_gt = draw_imgs_big_canvas(Lflgt) ;
figure; imshow(canvas_gt,cmap) ;
keyboard ;
%}


% % --------------------------------------------------------------%
%   If SUPERVISED, Initial foreground model.
% % --------------------------------------------------------------%
if OPT.SUPERVISED>0,

    % get gt info
    load(fullfile(DIR_WORK_HOME, FNAME_GROUP_INFO), 'GroupInf') ;
    [~, oi] = intersect({GroupInf(:).name}, obj_name) ;

    % bMatImFG (i,j): 1 or 0. whether FG i exists in image j.
    bMatImFG = sparse(double(max(GroupInf(oi).fgid)), nD) ;
    for i=1:nD,
        bMatImFG(intersect(GroupInf(oi).fgid, GroupInf(oi).labels{i}),i) = 1 ;
    end
    bMatImFG  = bMatImFG(GroupInf(oi).fgid,:) ;

    % divide the images into training and test sets
    [trainID OPT] = divide_train_test(bMatImFG, OPT) ;
    testID = 1:nD ; testID(trainID) = [] ;

    % get descriptors
    [descFGGMM descFGSPM] = get_descFG_from_gt(Lflgt(trainID), ...
        Lfsegm(trainID), Lffeat(trainID), OPT, GroupInf(oi).fgid) ;
    for i=1:OPT.ForeK+1, descFGGMM(i).imgID = trainID(descFGGMM(i).imgID) ; end
    if OPT.FG_MOD(2) > 0,
        for i=1:OPT.ForeK+1, descFGSPM(i).imgID = trainID(descFGSPM(i).imgID) ; end
    end

% % --------------------------------------------------------------%
%   If UNSUPERVISED,
% % --------------------------------------------------------------%
else,
    disp('Unsupervised version is not included in this toolbox.') ;

    % get descriptors
    [descFGGMM descFGSPM] = get_descFG_unsup(Lflgt, Lfsegm, ...
        Lffeat, OPT, []) ;
end

% Get initial GMM foreground model.
FG_GMM = run_GMM_FG_init (descFGGMM, OPT.FG_Ncomp) ;
clear descFGGMM ;

% Get initial SPM foreground model.
if OPT.FG_MOD(2) > 0,
    [FG_SPM] = run_SPM_FG_init (descFGSPM, OPT.SVM_TYPE) ;
    clear descFGSPM ;
else,
    FG_SPM = [] ;
end


% % --------------------------------------------------------------%
%       Iteration
% % --------------------------------------------------------------%

iter = 1 ;

% create the output folder
DIR_WORK_RES_ITER = fullfile(DIR_WORK_RES, obj_name, num2str(iter, '%.3d')) ;
if ~exist(DIR_WORK_RES_ITER), mkdir(DIR_WORK_RES_ITER) ; end

% % --------------------------------------------------------------%
%       REGION ASSIGNMENT
% % --------------------------------------------------------------%

% for each image
for ii=1:nD

    tic
    % load superpixels & segment
    load (Lfsegm{ii}, 'imgSP', 'imgSeg') ;
    % load descriptor file
    %load(Lffeat{ii}, 'descSP', 'sptHist', 'sptHistHSV') ;
    if OPT.BSIFTCOLOR>0,
        load(Lffeat{ii}, 'descSP', 'sptHistHSV') ;
        sptHist = sptHistHSV ; clear sptHistHSV ;
    else,
        load(Lffeat{ii}, 'descSP', 'sptHist') ;
    end

    % file name of output
    [~,fname] = fileparts(Lflimg{ii}) ;
    fres_ra = fullfile(DIR_WORK_RES_ITER, [fname TYPE_MAT]) ;

    %{
    % if already done, skip it.
    if exist(fres_ra),
        disp([obj_name ' : ' num2str(ii) ' is already done.']) ;
        continue ;
    end
    %}

% % --------------------------------------------------------------%
%       MAIN ALGORITHM
% % --------------------------------------------------------------%

    % get similarity between segments and components of Foregrounds
    [simSeg2Comp massSeg] = get_GMM_simSeg2Comp(imgSP, descSP, FG_GMM, OPT) ;

    if OPT.FG_MOD(2) > 0,
        % sptHist is histogram for SPs. Make histogram for segments.
        sptHistSeg = get_SPM_hist_seg(imgSP, sptHist) ;
    end

    % get candidate graph and candidate mst for each FG
    mstCandFg = cell(1, OPT.ForeK) ;
    graphCandFg = cell(1, OPT.ForeK) ;
    % GMM + SPM
    if OPT.FG_MOD(2) > 0,
        minmaxSPM = [] ;
        for k=1:OPT.ForeK+1,
            [mstCandFg{k} graphCandFg{k} minmax] = get_SPMGMM_MST_FG(imgSeg.adjMat, ...
                simSeg2Comp(:,k), massSeg(:,k), FG_SPM(k), sptHistSeg, ...
                OPT, Lflimg{ii}, Lfsegm{ii}) ;
            % minimum and maximum SPM scores (will be used for normalization)
            minmaxSPM = [minmaxSPM; minmax] ;
        end
    % GMM only
    else,
        for k=1:OPT.ForeK+1,
            [mstCandFg{k} graphCandFg{k}] = get_GMM_MST_FG(imgSeg.adjMat, ...
                simSeg2Comp(:,k), massSeg(:,k), Lflimg{ii}, Lfsegm{ii}) ;
        end
    end

    % generate candidates of each FG by Beam search
    setCandFg = cell(OPT.ForeK,1) ;
    valCandFg = cell(OPT.ForeK,1) ;
    % GMM + SPM
    if OPT.FG_MOD(2) > 0,
        for k=1:OPT.ForeK+1,
            [setCandFg{k} valCandFg{k}] = get_SPMGMM_candidates_beam_search( ...
                mstCandFg{k}, simSeg2Comp(:,k), massSeg(:,k), ...
                FG_SPM(k), sptHistSeg, minmaxSPM, OPT) ;
        end
    % GMM only
    else,
        for k=1:OPT.ForeK+1,
            [setCandFg{k} valCandFg{k}] = get_GMM_candidates_beam_search( ...
                mstCandFg{k}, simSeg2Comp(:,k), massSeg(:,k)) ;
        end
    end

    % Infer the most likely tree from candidate sets of K foregrounds
    [setCandFg valCandFg mstMLE] = infer_tree_cand(setCandFg, valCandFg, mstCandFg) ;

    % run region assignment by using dynamic programming
    [optSlnCand setCandFg valCandFg] = run_region_assignment_dp(setCandFg, valCandFg, mstMLE) ;

    % obtain segmentation output from optimal FG candidates
    [OptFGIdSeg OptFGIdSP segOutImg] = get_seg_from_opt_cand(optSlnCand, setCandFg, imgSP) ;

    % save the results
    save(fres_ra, 'trainID', 'mstCandFg', 'setCandFg', 'valCandFg', ...
        'mstMLE', 'OptFGIdSeg', 'OptFGIdSP', 'segOutImg') ;

% % --------------------------------------------------------------%
%       VISUALIZATION
% % --------------------------------------------------------------%

    if bDebug>0,
        % (1) image
        img = imread(Lflimg{ii}) ;
        % (2) oversegmented image
        imgOver = disp_draw_segment_by_color(imgSeg.segImg) ;
        % (3) segmentation image
        imgSegm = disp_draw_hsv_segment_on_img(Lflimg{ii}, segOutImg) ;
        % draw figure
        figure; subplot(1,3,1) ; imshow(img) ; title('Input image') ;
        subplot(1,3,2) ; imshow(imgOver) ; title('Oversegmets') ;
        subplot(1,3,3) ; imshow(imgSegm) ; title('Final segmentation');

        % cut image
        uniqLabels = unique(OptFGIdSeg) ;
        nlabels = length(uniqLabels) ;
        labelSet = cell(1,nlabels) ;
        for j=1:nlabels, labelSet{j} = find(OptFGIdSeg==uniqLabels(j)) ; end
        imgCut = disp_draw_segments_on_white_bg(img, imgSeg.segImg, labelSet);

        % (4) write cut images
        figure ; nFigRow = ceil(nlabels/4) ;
        for j=1:nlabels,
            subplot(nFigRow, 4, j); imshow(imgCut{j}) ;
        end
        keyboard ;
    end

    disp(['Done : ' num2str(ii) '/' num2str(nD) ' of ' obj_name '.']) ;
end

disp(['cosegmentation ' obj_name ' is done.']) ;

end



