function [libsvm_cl] = train_svm_1vsall (Y, data)
%
%   Train SVM :
%       See 'svm_one_vs_all.m' and 'kernel.m'
%       in 'sunfeature/SUN_source_code_v2/code/scene_sun/'.
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%
%   Y           : labels
%   data        : data
%
% --------
% OUTPUTS:
%
%   libsvm_cl   : SVM FG model
%
% ---------------
% Gunhee Kim
%

% get kernel (using histogram intersection)
K = vl_alldist2(data, 'kl1');

% learn svm
libsvm_cl = svmtrain(Y, double([(1:size(K,1))' K]), ...
    ['-q -t 4 -s 0 -w-1 1 -w1 ' num2str(length(find(Y==-1))/length(find(Y==1))) ' -c 1.0']) ;
ap = mean(libsvm_cl.sv_coef(Y(libsvm_cl.SVs) > 0)) ;
am = mean(libsvm_cl.sv_coef(Y(libsvm_cl.SVs) < 0)) ;
if ap < am
    % fprintf('svmflip: SVM appears to be flipped. Adjusting.\n') ;
    libsvm_cl.sv_coef  = - libsvm_cl.sv_coef ;
    libsvm_cl.rho      = - libsvm_cl.rho ;
end

% data of support vectors
libsvm_cl.SVdata = data(:, libsvm_cl.SVs) ;

%{
% the following is unnecessary
% test it on train
train_scores = libsvm_cl.sv_coef' * K(libsvm_cl.SVs, :) - libsvm_cl.rho ;
errs = train_scores .* Y' < 0 ;
err  = mean(errs) ;
%}

