function [InterNet] = gen_inter_net(LdescSpCLR, LdescSpTEX, ...
            WEIGHT_SCALE_INTER) 
%
%	Generate the Inter-network between the SPs of the input images 
%
% -------
% INPUTS:
%
%   descSpCLR          : Cell of color descriptors of SPs (N x [M x d]) 
%                         where N: # of images, M: # of SPs, 
%                         d: feat dimension (ex. rgb: 3)
%   descSpTEX           : Cell of texture descriptors of SPs. (same with 'descSpCLR')
%   WEIGHT_SCALE_INTER  : Sigma for Gaussian similarity
%
% --------
% OUTPUTS:
%
%   InterNet            
%       .num_eval       : number of evaluation points per image ([1 x N])
%       .mat            : Similarity network between evaluation points.
%
% -------------
%
% Gunhee Kim
%


SIM_THRES = 0.01 ;

num_set = length(LdescSpCLR) ;

InterNet = [] ;

% number of evaluation points per image
InterNet.num_eval = zeros(1,num_set) ;
for i=1:num_set,
    InterNet.num_eval(i) = size(LdescSpCLR{i},1) ;
end
eval_sind = [0 cumsum(InterNet.num_eval)] ;
num_all_eval = eval_sind(end) ;

InterNet.mat = cell(num_set, num_set) ;
for i=1:num_set, 
    for j=i+1:num_set, 
        % Color pairwise similarity
        if ~isempty(LdescSpCLR{i}),
            pair_dist_f = fn_weights_gauss(LdescSpCLR{i}, LdescSpCLR{j}, ...
                [], [], WEIGHT_SCALE_INTER) ;
            pair_dist_f(pair_dist_f<SIM_THRES) = 0 ;
        end
        % Texture pairwise similarity 
        if ~isempty(LdescSpTEX{i}),
            pair_dist_t = fn_weights_gauss(LdescSpTEX{i}, LdescSpTEX{j}, ...
                [], [], WEIGHT_SCALE_INTER) ;
            pair_dist_t(pair_dist_t<SIM_THRES) = 0 ;
            pair_dist_f = pair_dist_f + 0.5*pair_dist_t ;
        end

        min_ind = min(i,j) ;
        max_ind = max(i,j) ;
        InterNet.mat{min_ind,max_ind} = full(pair_dist_f) ;
    end
end
clear pair_dist_f pair_dist_t tmpNet rr cc vv ;

end
