function [v] = get_potential_lineq_power(cond_mat, pos_pole, neg_pole, initV)
%
%	Compute Potential
%   Solution by direct linear method.
%   See the Page 21 of 'Random Walks and electric networks'
%       by  P.G. Doyle and J.L. Snell
%
% -------
% INPUTS:
%
%	cond_mat        : conductance network ([N x N]) 
%                     where N is the number of vertices
%   pos_pole        : nodes attached to sources
%   neg_pole        : nodes attached to ground
%
% --------
% OUTPUTS:           
%   v               : potential
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%


% M: number of sets
[N M] = size(cond_mat) ;

% tolerance for sum
CONST_TOL = M * 10^-6 ;

% number of iterations for maximum voltage increase
CONST_ITER_VOL = 50000 ;

if nargin<5 || isempty(initV), 
    v = zeros(N,1) ;
    v(pos_pole) = 1 ;
else
    v = initV ; 
end

pos_pole = unique(pos_pole) ;

pre_sum_v = sum(v) ;
for i=1:CONST_ITER_VOL,

    % power iteration
    v = cond_mat * v ;
    v(pos_pole) = 1 ;
    v(end,:) = 0 ;    

    sum_v = sum(v) ;
    
    if (sum_v - pre_sum_v) < CONST_TOL,
        break ;
    end
    pre_sum_v = sum_v ;

end
