function [bSingular] = is_matrix_singular(A, b)
%
%	Check whether A is singular or not. 
%
% -------
% INPUTS:
%   A :             input matrix
%   b :             (optional) x = A \ b
%
% --------
% OUTPUTS:
%   bSingular :     1 if A is singular. 0 otherwise. 
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

nA = size(A,1) ;

if nargin < 2, 
    b = ones(nA,1) ;
end

% There are a lot of different ways to check wehther A is sinular
% or not. 

% rank() and cond() is too slow. 

bSingular = 0 ;

% (1) Solve linear system. If there's NAN in X, it's singular. 

x = A \ b ;

if ~isempty(find(isnan(x))), 
    bSingular = 1 ;     
    disp('L is sinular!') ;
end

return


% (2) Another way is use det(). This is faster than (1). 
% If its output is too small or nan, it's singular

detA = det(A) ;

if detA<10^-6,
    bSingular = 1 ;
    disp('L is sinular!') ;
elseif isnan(detA),
    bSingular = 1 ;
    disp('L is sinular!') ;
end


end
