function [clusterID probMat] = run_cluster_from_rankset(adj_mat, srcInd, clID)
%
%	Clustering from Source Points (ie. ranked items).
%
% -------
% INPUTS:
%
%	adj_mat 		: Similarity network ([N x N])
%   srcInd          : Source points. ([1 x K])
%   clID            : cluster IDs of source points [1 x K] 
%
% --------
% OUTPUTS:
%
%   clusterID       : cluster id of each point ([N x 1])
%   probMat         : probability matrix ([N x K])
%
% -------------
%
% Gunhee Kim
%

SMALL_NUM = 0.01 ;
TINY_NUM = 0.0001 ;

% Laplacian
L = diag(sum(adj_mat,2)) - adj_mat ;

% number of points
N=length(L);
% number of clusters
K = max(clID) ;

% Px1 Vector specifying the "boundary" nodes
srcMat = zeros(K, K) ;
for i=1:K,
    s_ind = find(clID==i) ;
    srcMat(i,s_ind) = 1 ;
end

% indices that are not sources
plainInd = 1:N ;
plainInd(srcInd) = [] ;

% Find RHS
b = -L(plainInd,srcInd) * srcMat ;

% Check whether if A is singular,
% singular_ind = find(isnan(sum(x,2))) ;

% Solve linear system
% Check whether if A is singular,
if is_matrix_singular(L(plainInd,plainInd), b)
    % (1) Simple Tikhonov regularization
    x = tikhonov_regularization(L(plainInd,plainInd)) \ b ; 
else
    x = L(plainInd,plainInd) \ b ; 
end

% x should be larger than 0. 
x(x<0) = 0 ;

% Probability
probMat = zeros(N,K) ;
probMat(srcInd,:) = srcMat ;
probMat(plainInd,:) = x ;

% cluster ID
[foo clusterID] = max(probMat,[],2) ;

%{
% assign the largest clust to singular elements
if ~isempty(singular_ind), 
    [max_val max_id] = max(histc(clusterID,1:clID)) ;
    clusterID(singular_ind) = max_id ;
end
%}

% if prob = 0, assign cluster ID by K+1. 
clusterID(sum(probMat,2)<TINY_NUM) = K+1 ;
%clusterID(isnan(sum(probMat,2))) = K+1 ;

probMat(probMat<0) = 0 ;
