function [GraphImg] = disp_draw_segm_adj_net(Img, adj_net, cntSeg, segm_id_set)
%
%   Draw segment connectivity graph on image
%
% -------
% INPUTS:
%
%   Img             : input image.
%   adj_net         : connectivity graph between segments
%   cntSeg          : center of SP (ImgSeg)
%   segm_id_set     : segment id of each SP (ImgSeg)
%   clusterImg      : superpixel image.
%   BdSegImg        : boundary image
%   bDraw           : 1 = draw the output, 0: do not draw
%   svName          : if it is empty, do not save the output.
%                     if it is a filename, save the output.
%
% --------
% OUTPUTS:
%
%   GraphImg        : segment connectivity graph on image
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%

adj_net(adj_net>0) = 1 ;

% find center SP of each segment
seg_cnt_sp = zeros(1,length(adj_net)) ;
for v=1:length(adj_net),
    tmp_seg_cnt = cntSeg(segm_id_set{v},:) ;
    tmp_dist = tmp_seg_cnt - repmat(mean(tmp_seg_cnt), [size(tmp_seg_cnt,1) 1]) ;
    [min_val min_ind] = min(sum(tmp_dist.^2,2)) ;
    seg_cnt_sp(v) = segm_id_set{v}(min_ind) ;
end

tmp_mat = tril(adj_net,-1) ;
[tmpr tmpc tmpv] = find(tmp_mat) ;
nnze = length(tmpr) ;

% get graph connectivity
cnt_list = [cntSeg(seg_cnt_sp(tmpr),1) ...
    cntSeg(seg_cnt_sp(tmpc),1) cntSeg(seg_cnt_sp(tmpr),2) ...
    cntSeg(seg_cnt_sp(tmpc),2)] ;

% draw connectivity
GraphImg = mex_draw_thick_lines_on_img(Img, cnt_list, ...
    4*ones(nnze,1), 255*ones(nnze,3)) ;

% no input draw it. 
if nargout<1,
    figure; imshow(GraphImg) ;    
end

end
