function [GraphImg] = disp_draw_segm_adj_net_thinning(Img, adj_net, seg_im)
%
%   Draw segment connectivity graph on image
%
% -------
% INPUTS:
%
%   Img             : input image.
%   adj_net         : connectivity graph between segments
%   seg_im          : segment image
%
% --------
% OUTPUTS:
%
%   GraphImg        : segment connectivity graph on image
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%

adj_net(adj_net~=0) = 1 ;

%{
% find center SP of each segment
seg_cnt_sp = zeros(1,length(adj_net)) ;
for v=1:length(adj_net),
    tmp_seg_cnt = cntSeg(segm_id_set{v},:) ;
    tmp_dist = tmp_seg_cnt - repmat(mean(tmp_seg_cnt), [size(tmp_seg_cnt,1) 1]) ;
    [min_val min_ind] = min(sum(tmp_dist.^2,2)) ;
    seg_cnt_sp(v) = segm_id_set{v}(min_ind) ;
end
%}

% find center SP of each segment
seg_cnt = zeros(length(adj_net),2) ;
for v=1:length(adj_net),
    vim = seg_im ;
    vim(vim~=v) = 0 ;
    vim(vim==v) = 1 ;
    % thinning the segment
    bivim = bwmorph(vim,'thin',Inf); 
    
    %figure; imshow(bivim) ;
    % finding the biggest component and ignore the other components
    ccvim = bwconncomp(bivim);
    npixcc = zeros(1, ccvim.NumObjects) ;
    for c=1:ccvim.NumObjects,
        npixcc(c) = length(ccvim.PixelIdxList{c}) ;
    end
    [max_val max_ind] = max(npixcc) ;
    for c=1:ccvim.NumObjects,
        if c==max_ind, continue ; end
        bivim(ccvim.PixelIdxList{c}) = 0 ;
    end
    %figure; imshow(bivim) ;

    [rcnt ccnt] = find(bwmorph(bivim,'branchpoints')) ;
    if isempty(rcnt), [rcnt ccnt] = find(bivim) ; end
    tmp_seg_cnt = mean([rcnt ccnt]) ;
    % find the closest thinning line to the mean
    tmp_dist = [rcnt ccnt] - repmat(tmp_seg_cnt, [size(rcnt,1) 1]) ;
    [min_val min_ind] = min(sum(tmp_dist.^2,2)) ;
    seg_cnt(v,:) = [rcnt(min_ind) ccnt(min_ind)] ;
    %{
    tmp_cnt = sortrows([rcnt ccnt]) ;
    seg_cnt(v,:) = tmp_cnt(round(size(tmp_cnt,1)/2),:) ;
    %}
end

tmp_mat = tril(adj_net,-1) ;
[tmpr tmpc tmpv] = find(tmp_mat) ;
nnze = length(tmpr) ;

%{
tmpr = tmpr(1:10) ;
tmpc = tmpc(1:10) ;
tmpv = tmpv(1:10) ;
nnze = 10 ;
%}
%{
% get graph connectivity
cnt_list = [cntSeg(seg_cnt_sp(tmpr),1) ...
    cntSeg(seg_cnt_sp(tmpc),1) cntSeg(seg_cnt_sp(tmpr),2) ...
    cntSeg(seg_cnt_sp(tmpc),2)] ;
%}

% get graph connectivity
cnt_list = [seg_cnt(tmpr,1) seg_cnt(tmpc,1) seg_cnt(tmpr,2) ...
    seg_cnt(tmpc,2)] ;

% draw connectivity
GraphImg = mex_draw_thick_lines_on_img(Img, cnt_list, ...
    5*ones(nnze,1), 255*ones(nnze,3)) ;

% no input draw it. 
if nargout<1,
    figure; imshow(GraphImg) ;
end

end
