function [img_out] = disp_get_color_img_from_ids(img)
%
%   Change ID image to color image
%
% -------
% INPUTS:
%
%   img             : input image (integer [nr x nc]) 
%
% --------
% OUTPUTS:
%
%   img_out         : output image ([nr x nc x 3]) 
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%

% size of image
[nr nc] = size(img) ;

img = double(img) ;

uniq_id = unique(img(:)) ;

% if the id image contains 0, plus all by one
if ~isempty(find(uniq_id==0)), 
    img = img + 1 ;
end

% color map
cmap = colormap(lines(max(uniq_id)+1)) ;

% image output
img_out = zeros([nr, nc 3]) ;
for i=1:3,
    img_out(:,:,i) = reshape(cmap(img,i),[nr nc]) ;
end

end
