function [clImg] = fill_up_boundary(clImg, pref_order)
%
%   Sometimes, clusterImage have regions of 0, which indicate
%   boundaries. We fill them up with nearby clusterID.
%
% -------
% INPUTS:
%
%   clImg           : clusterImage with boundary==0
%   pref_order      : preference between clusters
%
% --------
% OUTPUTS:
%
%   clusterImg      : Output segment image.
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%

numCL = length(unique(clImg(:))) ;

%{
if nargin<2,
    pref_order = 1:max(clImg(:)) ;
end
%}

MAX_ITER = 5 ;

[nr nc] = size(clImg) ;

iter = 0 ;
while 1,

    iter = iter + 1 ;

    [rz cz] = find(clImg==0) ;

    if isempty(rz), break; end

    rzm = rz - 1 ; rzp = rz + 1 ;
    rzm(rzm==0) = rz(rzm==0) ;
    rzp(rzp>nr) = rz(rzp>nr) ;
    czm = cz - 1 ; czp = cz + 1 ;
    czm(czm==0) = cz(czm==0) ;
    czp(czp>nc) = cz(czp>nc) ;

    rzs = [rzm rzm rzm rz  rz  rzp rzp rzp] ;
    czs = [cz  czm czp czm czp cz  czm czp] ;

    sz = sub2ind([nr nc], rzs, czs) ;
    label_cl = clImg(sz) ;

    if iter<MAX_ITER,
        % find most frequent labels
        label_final = mode(-label_cl,2) ;
        label_final = -label_final ;
    else
        label_final = max(label_cl,[], 2) ;
    end

    %{
    label_final = zeros(size(sz,1),1) ;
    for j=numCL:-1:1,
        [rf cf] = find(label_cl==pref_order(j)) ;
        rf = unique(rf) ;
        label_final(rf) = pref_order(j) ;
    end
    %}

    clImg(sub2ind([nr nc], rz, cz)) = label_final ;
end

end
