function [img] = writeBoundingBox(img, bb, score, cmap)
%
%   Visualices and image and a set of bounding boxes
%
% -------
% INPUTS:
%
%   img             : input image
%   bb              : bounding boxes (# of BB x [xmin ymin xmax ymax])
%   score           : thickness
%   colors          : colors of bounding boxes
%
% --------
% OUTPUTS:
%
%   img             : output image
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%


% thickness of bounding boxes
Nbb = size(bb,1);
if nargin<3,
    score = 3*ones(Nbb,1);
end
score = round(score) ;

% color assignment
if nargin<4,
    cmap = fn_VOClabelcolormap(256) ;
end


% check whther image path or image itself.
if ischar(img), 
    img = imread(img) ; 
end



[imy imx imch] = size(img) ;
img = reshape(img,[imy*imx imch]) ;

% bounding box drawing
for i=Nbb:-1:1
	range_min = -floor(score(i)/2) ;
	range_max = score(i) -1 + range_min ;
	line_x =[]; line_y =[]; 
	for j=range_min:range_max,

		new_bb(1) = bb(i,1)+j ; 
		new_bb(2) = bb(i,2)+j ;
		new_bb(3) = bb(i,3)-j ;
		new_bb(4) = bb(i,4)-j ;

		len_x = new_bb(3) - new_bb(1)+1 ;
		len_y = new_bb(4) - new_bb(2)+1 ;

		line_x = [line_x new_bb(1):new_bb(3)]; 
		line_y = [line_y new_bb(2)*ones(1,len_x)];

		line_x = [line_x new_bb(1):new_bb(3)]; 
		line_y = [line_y new_bb(4)*ones(1,len_x)];  

		line_x = [line_x new_bb(1)*ones(1,len_y)];
		line_y = [line_y new_bb(2):new_bb(4)]; 

		line_x = [line_x new_bb(3)*ones(1,len_y)];
		line_y = [line_y new_bb(2):new_bb(4)]; 

		r_idx = unique([find(line_x<1) find(line_x>imx) find(line_y<1) find(line_y>imy)]) ;
		line_x(r_idx) = [] ;
		line_y(r_idx) = [] ;
	end

	pnt_set = unique(sub2ind ([imy imx], line_y, line_x)) ;
	if imch ==3, 
		img(pnt_set,1) = cmap(i,1) ;
		img(pnt_set,2) = cmap(i,2) ;
		img(pnt_set,3) = cmap(i,3) ;
	else
		img(pnt_set) = cmap(i,1) ;
	end
end
img = reshape(img,[imy imx imch]) ;
