function blurImg = run_bilateralFilt(img)
%
%  Run bilateral filtering
%
% -------
% INPUTS:
%
%   img :       Input image
%
% --------
% OUTPUTS:
%
%   blurImg :   blurred image
%
% ---------------
% Gunhee Kim
%

nCh = size(img,3) ;

% For color image, we first change RGB into Lab color space, and
% then apply bilateral filterin in each channel.
% see the justification at
% http://homepages.inf.ed.ac.uk/rbf/CVonline/LOCAL_COPIES/MANDUCHI1/Bilateral_Filtering.html
if nCh > 1,
    blurImg = zeros(size(img)) ;
    img  = colorspace('Lab<-RGB', img);
    for i=1:nCh,
        blurImg(:,:,i) = bilateralFilter(img(:,:,i)) ;
    end
    blurImg = colorspace('RGB<-Lab', blurImg);

% for grayscale iamge
else,
    blurImg = bilateralFilter(img) ;
end

