function script_cosegment ()
%
%  Cosegment multiple images
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%
% --------
% OUTPUTS:
%
% ---------------
% Gunhee Kim
%

close all; warning off;

% % --------------------------------------------------------------%
%     Add Pathes
% % --------------------------------------------------------------%

if ~exist('fn_dist_l2'),        addpath('div_rank') ;   end
if ~exist('run_superpixel'),    addpath('segment') ;    end
if ~exist('gen_inter_net'),     addpath('coseg') ;      end
if ~exist('disp_draw_segment_by_color'), addpath('draw') ; end

% add TurboPixel path if it exists
lib_path = 'lib/TurboPixels/';
if exist(lib_path, 'dir'),
    if ~exist('get_speed_based_on_gradient'),   addpath(lib_path) ; end
    if ~exist('computeExtensionFields2d'),      addpath([lib_path 'lsmlib']) ; end
end
% add vlfeat path
lib_path = 'lib/vlfeat-0.9.14/';
if exist(lib_path, 'dir'),
    if ~exist('vl_setup'), run([lib_path 'toolbox/vl_setup']) ; end
    if ~exist('run_bilateralFilt'), addpath('lib/bilateralFilt') ; end
end


% % --------------------------------------------------------------%
%     Parameters
% % --------------------------------------------------------------%

% 1: Draw figure. 0: Do not.
bDraw = 1 ;
% 1: Save the result as a file. 0: Do not.
bSave = 0 ;

% Number of segments
OPTS.K = 3 ;
% scale of gaussian weights
OPTS.WGT_SCALE_PCT      = 0.25 ;
OPTS.WGT_SCALE_INTER      = 0.6 ;

% Set options
% see the default options in 'opts_single_seg.m'
OPTS = opts_cosegment(OPTS) ;

% % --------------------------------------------------------------%
%     Load an image, Extract superpixels (SP), and
%     Get descriptor of each SP.
% % --------------------------------------------------------------%

% load and resize an image
Lflimg = {'pics/1_16_s.bmp', 'pics/1_19_s.bmp', 'pics/1_20_s.bmp'} ;


nLflimg = length(Lflimg) ;
LspImg = cell(1,nLflimg) ;
LlocNet = cell(1,nLflimg) ;
LEvalSet = cell(1,nLflimg) ;
LZ_SP = cell(1,nLflimg) ;
LdescEvalCLR = cell(1,nLflimg) ;
LdescEvalTEX = cell(1,nLflimg) ;

for h=1:nLflimg,
% resize the image.

    img = imread(Lflimg{h}) ;
    [nr nc nch] = size(img) ;
    imsize_org = [nr nc nch] ;
    im_res_rate = min(OPTS.MAX_IMG_SIZE ./ [nr nc]) ;
    img = imresize(img,im_res_rate) ;

    % Generate Superpixels
    % spImg: segmet images,
    % AdjMat: Neighborhood matrix btw segments
    % SP_MODE: Used segmentation method
    % cntSeg: center point of each segment.
    [spImg, AdjMat, cntSeg] = run_superpixel(img, OPTS.NUM_SP, OPTS.SP_MODE) ;

    % Extract color features from each SP
    % descSpCLR: RGB, HSV, or CIELab descriptor of super-pixel
    [descSpCLR descSpTEX] = extract_feat_segment(img, spImg);

    %construct intra-image network of each image.
    [locNet Z_SP] = gen_intra_net(descSpCLR, descSpTEX,...
            cntSeg, OPTS.C_INTRA_NGH, OPTS.WGT_SCALE_PCT, OPTS.CONST_Z) ;

    % generate evaluation points (where sources can be placed).
    [EvalSet aggloClID] = gen_eval_list(descSpCLR, locNet, ...
            spImg, AdjMat, OPTS.NUM_EVAL_PNT, OPTS.CLUST_MIN) ;

    %{
    % (DEBUG) display the evaluation points.
    [label_img] = disp_draw_intra_agglo_real(img, spImg, EvalSet, 1) ;
    %}

    % Descriptor values of evaluation points.
    if ~isempty(descSpCLR), LdescEvalCLR{h} = descSpCLR(EvalSet,:) ; end
    if ~isempty(descSpTEX), LdescEvalTEX{h} = descSpTEX(EvalSet,:) ; end

    % save the pre-processing results.
    LspImg{h} = spImg ;
    LlocNet{h} = locNet ;
    LEvalSet{h} = EvalSet ;
    LZ_SP{h} = Z_SP ;
end
clear descSpCLR descSpTEX AdjMat cntSeg aggloClID ;

save('output/tmp.mat', 'LspImg', 'LlocNet', 'LEvalSet', ...
     'LdescEvalCLR', 'LdescEvalTEX', 'LZ_SP') ;

load('output/tmp.mat', 'LspImg', 'LlocNet', 'LEvalSet', ...
     'LdescEvalCLR', 'LdescEvalTEX', 'LZ_SP') ;


% % --------------------------------------------------------------%
%       Generate the Inter-network between the Evaluation Points
%       of the input images
% % --------------------------------------------------------------%

[InterNet] = gen_inter_net(LdescEvalCLR, LdescEvalTEX, ...
            OPTS.WGT_SCALE_INTER) ;

clear LdescSpFeat LdescSpTEX ;

% % --------------------------------------------------------------%
%       Run Diversity Ranking and Clustering
% % --------------------------------------------------------------%

% size of image set.
num_set = length(LlocNet) ;

% conductance to dummy ground
% Z =  constZ*mean(LlocNet{1}(LlocNet{1}>0)) ;

% Each SP is weighted by its number of pixel
LmassSP = cell(1,num_set) ;
for h=1:num_set
    LmassSP{h} = histc(LspImg{h}(:), [0:size(LlocNet{h},1)]) ;
    LmassSP{h}(1) = [] ;
end

% get diverse and central rank set.
% (1) naive greedy
[Lrank_set Lrank_set_val] = run_coseg_naive_greedy(LlocNet, ...
       OPTS.K, OPTS.LIN_MODE, LZ_SP, LEvalSet, LmassSP, InterNet) ;
save ('output/tmp_rank_set.mat', 'Lrank_set') ;

% load ('output/tmp_rank_set.mat', 'Lrank_set') ;

% run clustering from rank set.
Lcluster_id = cell(1,num_set) ;
for h=1:num_set,
    Lcluster_id{h} = run_cluster_from_rankset(LlocNet{h}, Lrank_set{h}(1:OPTS.K), 1:OPTS.K) ;
end


% % --------------------------------------------------------------%
%     Visualization
% % --------------------------------------------------------------%

for h=1:num_set,

num_rank_set = length(Lrank_set{h}) ;

% (1) load an image
parsePnt = strfind(Lflimg{h},'/') ;
parsePnt = parsePnt(end) ;
filename = Lflimg{h}(parsePnt+1:end) ;
fileext =  Lflimg{h}(end-3:end) ;

Img = imread(Lflimg{h}) ;
[nr nc] = size(LspImg{h}) ;
nch = size(Img,3) ;
Img = imresize(Img, [nr nc]) ;

% (2) draw the temperature image for each cluster
% number of vertex
numSP = max(LspImg{h}(:)) ;


% get cluster image
[clusterImg] = get_segm_img_from_sp(LspImg{h}, Lcluster_id{h}) ;

% get cluster boundary image
[BdSegImg] = disp_get_segment_boundary(clusterImg) ;
if bSave,
    outfpre = fullfile('output', [strrep(filename, fileext, ''), '_' , ...
                num2str(OPTS.K,'%.2d')]) ;
end

% (1) draw original image
if bDraw, figure; imshow(Img) ; end
if bSave, imwrite(Img, [outfpre '.png']) ; end


% (2) draw clusters
[ImgSegm] = disp_draw_segment_by_color(clusterImg, bDraw) ;
if bSave, imwrite(ImgSegm, [outfpre '_segm' '.png']) ; end

%{
% (3) draw clusters on the photo
[ImgSegmHSV] = disp_draw_hsv_segment_on_img(Img, clusterImg, BdSegImg, bDraw) ;
if bSave, imwrite(ImgSegmHSV, [outfpre '_segm_hsv' '.png']) ; end
%  [strrep(filename, '.jpg', ['_clust_' num2str(num_rank_set) '.png'])]) ;


% (4) draw boundaries
[ImgBdSeg] = disp_draw_imgs_segment_boundary(Img, BdSegImg, bDraw) ;
if bSave, imwrite(ImgBdSeg, [outfpre '_bd' '.png']) ; end
%  [strrep(filename, '.jpg', ['_segbnd_' num2str(num_rank_set) '.png'])]) ;


% compute temperature contributed by each source.
[T_i] = disp_get_potential_per_source(LlocNet{h}, Lrank_set{h}, LZ_SP{h}) ;

% (5) draw temperature
[ImgTemp] = disp_draw_temperature(Lflimg{h}, LspImg{h}, T_i, bDraw) ;
if bSave, for ii=1:length(ImgTemp),
    imwrite(ImgTemp{ii}, [outfpre '_temp_' num2str(ii,'%.2d') '.png']) ;
end, end

% (6) draw segmented real images.
[ImgCut] = disp_draw_segments_on_white_bg(Img, clusterImg, {[3 4]}, bDraw) ;
if bSave, imwrite(ImgCut{1}, [outfpre '_cut_' num2str(1, '%.2d') '.png']) ;end
%if bSave, for ii=1:length(ImgCut),
%    imwrite(ImgCut{ii}, [outfpre '_cut_' num2str(ii, '%.2d') '.png']) ;
%end, end
%}
end

end

