function script_segment_single ()
%
%  Segment a single image
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%
% --------
% OUTPUTS:
%
% ---------------
% Gunhee Kim
%

%close all; warning off;

% % --------------------------------------------------------------%
%     Add Pathes
% % --------------------------------------------------------------%

if ~exist('fn_dist_l2'),            addpath('div_rank') ;   end
if ~exist('run_superpixel'),        addpath('segment') ;    end
if ~exist('disp_get_cluster_img'),  addpath('draw') ;       end

% add TurboPixel path if it exists
lib_path = 'lib/TurboPixels/';
if exist(lib_path, 'dir'),
    if ~exist('get_speed_based_on_gradient'),   addpath(lib_path) ; end
    if ~exist('computeExtensionFields2d'),      addpath([lib_path 'lsmlib']) ; end
end
% add vlfeat path
lib_path = 'lib/vlfeat-0.9.14/';
if exist(lib_path, 'dir'),
    if ~exist('vl_setup'), run([lib_path 'toolbox/vl_setup']) ; end
    if ~exist('run_bilateralFilt'), addpath('lib/bilateralFilt') ; end
end


% % --------------------------------------------------------------%
%     Parameters
% % --------------------------------------------------------------%

% 1: Draw figure. 0: Do not.
bDraw = 1 ;
% 1: Save the result as a file. 0: Do not.
bSave = 0 ;

% Number of segments
OPTS.K = 4 ;
% scale of gaussian weights
OPTS.WGT_SCALE_PCT = 0.5 ;

% Set options
% see the default options in 'opts_single_seg.m'
OPTS = opts_single_seg(OPTS) ;

% input image
flimg = 'pics/17_9_s.bmp' ;

% % --------------------------------------------------------------%
%     Load an image, Extract superpixels (SP), and
%     Get descriptor of each SP.
% % --------------------------------------------------------------%


% load and resize an image.
img = imread(flimg) ;
[nr nc nch] = size(img) ;
imsize_org = [nr nc nch] ;
im_res_rate = min(OPTS.MAX_IMG_SIZE ./ [nr nc]) ;
img = fn_imresize(img,im_res_rate) ;

% Generate Superpixels
% spImg: superpixel images,
% AdjMat: Neighborhood matrix btw segments
% SP_MODE: Used segmentation method
% cntSeg: center point of each segment.
[spImg, AdjMat, cntSeg] = run_superpixel(img, OPTS.NUM_SP, OPTS.SP_MODE) ;

% Extract color features from each SP
% descSpCLR: RGB, HSV, or CIELab descriptor of super-pixel
[descSpCLR descSpTEX] = extract_feat_segment(img, spImg);

% construct intra-image network of each image.
[locNet Z_SP] = gen_image_graph(descSpCLR, descSpTEX,...
        cntSeg, OPTS.C_INTRA_NGH, OPTS.WGT_SCALE_PCT, OPTS.CONST_Z) ;

% generate evaluation points (where sources can be placed).
[EvalSet aggloClID] = gen_eval_list(descSpCLR, locNet, ...
        spImg, AdjMat, OPTS.NUM_EVAL_PNT, OPTS.CLUST_MIN) ;

%{
% (DEBUG) display the evaluation points
if bDraw,
    [label_img] = disp_draw_intra_agglo_real(img, spImg, EvalSet, 1) ;
    [clusterImg] = get_segm_img_from_sp(spImg, aggloClID) ;
    [ImgSegm] = disp_draw_segment_by_color(clusterImg, bDraw) ;
end
if bSave, imwrite(label_img, [outfpre '_sp_eval' '.png']) ; end
%}

clear descSpCLR descSpTEX cntSeg AdjMat ;

% % --------------------------------------------------------------%
%       Run Diversity Ranking and Clustering
% % --------------------------------------------------------------%

% conductance to dummy ground
% Z =  constZ*mean(locNet(locNet>0)) ;

% Each SP is weighted by its number of pixel
massSP = histc(spImg(:), [0:size(locNet,1)]) ;
massSP(1) = [] ;


% get diverse and central rank set.
% I implemented two different ways
% (1) Lazy greedy
[segm_cnt segm_cnt_val] = run_div_rank_lazy_greedy(locNet, OPTS.K, ...
        OPTS.LIN_MODE, Z_SP, massSP, EvalSet, OPTS.AUTOK) ;

%{
% (2) naive greedy
[segm_cnt segm_cnt_val] = run_div_rank_naive_greedy(locNet, OPTS.K, ...
        OPTS.AUTOK, OPTS.LIN_MODE, Z_SP, EvalSet, massSP) ;
%}
K = length(segm_cnt) ;

% run clustering from rank set.
[cluster_id] = run_cluster_from_rankset(locNet, segm_cnt, 1:K) ;

% get cluster image
[clusterImg] = get_segm_img_from_sp(spImg, cluster_id) ;


% % --------------------------------------------------------------%
%     Visualization
% % --------------------------------------------------------------%

num_segm_cnt = length(segm_cnt) ;

% (1) load an image
parsePnt = strfind(flimg,'/') ;
parsePnt = parsePnt(end) ;
filename = flimg(parsePnt+1:end) ;
fileext =  flimg(end-3:end) ;

Img = imread(flimg) ;
[nr nc] = size(spImg) ;
nch = size(Img,3) ;
Img = imresize(Img, [nr nc]) ;

% (2) draw the temperature image for each cluster
% number of vertex
numSP = max(spImg(:)) ;

% get cluster boundary image
if bDraw, [BdSegImg] = disp_get_segment_boundary(clusterImg) ; end

if bSave,
    outfpre = fullfile('output', [strrep(filename, fileext, ''), ...
        '_' , num2str(K,'%.2d')]) ;
end

%{
% (1) draw original image
if bDraw, figure; imshow(Img) ; end
if bSave, imwrite(Img, [outfpre '.png']) ; end

% (2) draw clusters
[ImgSegm] = disp_draw_segment_by_color(clusterImg, bDraw) ;
if bSave, imwrite(ImgSegm, [outfpre '_segm' '.png']) ; end
%}

% (3) draw clusters on the photo
if bDraw, [ImgSegmHSV] = disp_draw_hsv_segment_on_img(Img, clusterImg, BdSegImg, bDraw) ; end
if bSave, imwrite(ImgSegmHSV, [outfpre '_segm_hsv' '.png']) ; end
%  [strrep(filename, '.jpg', ['_clust_' num2str(num_segm_cnt) '.png'])]) ;

%{

% (4) draw boundaries
if bDraw, [ImgBdSeg] = disp_draw_imgs_segment_boundary(Img, BdSegImg, bDraw) ; end
if bSave, imwrite(ImgBdSeg, [outfpre '_bd' '.png']) ; end
%  [strrep(filename, '.jpg', ['_segbnd_' num2str(num_segm_cnt) '.png'])]) ;

% (5) draw temperature
if bDraw,
    % compute temperature contributed by each source.
    [T_i] = disp_get_potential_per_source(locNet, segm_cnt, Z_SP) ;
    [ImgTemp] = disp_draw_temperature(flimg, spImg, T_i, bDraw) ;
end
if bSave, for ii=1:length(ImgTemp),
    imwrite(ImgTemp{ii}, [outfpre '_temp_' num2str(ii) '.png']) ;
end, end

% (6) draw segmented real images.
[ImgCut] = disp_draw_segments_on_white_bg(Img, clusterImg, [], bDraw) ;
if bSave, for ii=1:length(ImgCut),
    imwrite(ImgCut{ii}, [outfpre '_cut_' num2str(ii) '.png']) ;
end, end
%}


end

