function script_toy_diverse_ranking_gaussian() 
%
%	A toy example of diversity ranking - 
%   rand background + 3 Normal dist with different centers. 
%
% -------
% INPUTS:
%
%
% --------
% OUTPUTS:
%
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

% % --------------------------------------------------------------%
%     Add Pathes
% % --------------------------------------------------------------%

if ~exist('fn_dist_l2'),
    addpath('div_rank') ;
end


% % --------------------------------------------------------------%
%     Data Generation
% % --------------------------------------------------------------%

close all ; 
warning off ;

%{
range_XY = 1 ;
% rand background + 3 Normal dist with different centers. 
X = [] ;
X = range_XY* rand(100,2) ;
X = [X; mvnrnd(range_XY*[0.4 0.3], 0.002*range_XY*eye(2),90)];
X = [X; mvnrnd(range_XY*[0.8 0.75],0.003*range_XY*eye(2),60)];
X = [X; mvnrnd(range_XY*[0.3 0.7], 0.005*range_XY*eye(2),50)];

figure; plot(X(:,1),X(:,2),'o');
save ('data/data_toy_gaussian.mat', 'X') ;
keyboard;
%}

% load the data
load ('data/data_toy_gaussian.mat', 'X') ;

% Centralize and scale the data
X = X - repmat(mean(X),size(X,1),1);
X = X/max(max(abs(X)));

% % --------------------------------------------------------------%
%     Parameters
% % --------------------------------------------------------------%

% number of cluster centers
K = 3 ; 

% Select a method to solve linear eq Ax=b.
% 0: incremental inverse. 
% 1: LU decomposition.
% 2: naive backslash. 
% 3: power iteration
LIN_MODE = 0 ;

% The contant for ground conductance
constZ = 0.1 ;

% Scale to use for affinity matrix
scale = 0.15 ;
% K-neighbor for local scaling
K_LOCAL_SCALE = 7 ;

% Distances less than this threshold are set to 0. 
ThresDist = 0.0001 ;

% save the results ?
bSave = 0 ;

% % --------------------------------------------------------------%
%     Generate Affinity Matrix
% % --------------------------------------------------------------%

nX = size(X,1) ;

% Pairwise square of l2 distances. 
D = fn_dist_l2_sqrt(X, X) ;

% Build affinity matrices

% (1) Standard affinity matrix (single scale)
A = exp(-D/(scale^2));       

% (2) Locally scaled affinity matrix
% [D_LS,A_LS,LS] = scale_dist(D,K_LOCAL_SCALE); 
% A = fn_local_scaling(D, K_LOCAL_SCALE) ;

% Distances less than ThresDist are set to 0. 
A(A<ThresDist) = 0 ;

% Set diagonal elements to zeros
A(eye(size(A))~=0)=0 ;
clear D ;

% sparsify by k-nn
%A = get_knn_matrix(A, round(2*log(nX))) ;

% % make the graph to have a single component
% sim_mat = fn_make_single_component(sim_mat) ;

% conductance to dummy ground
Z =  constZ*mean(A(A>0)) ;

% % --------------------------------------------------------------%
%     Run Diversity Ranking and Clustering
% % --------------------------------------------------------------%

% get diverse and central rank set.
% I implemented two different ways 
% (1) Lazy greedy
 [rank_set rank_set_val] = run_div_rank_lazy_greedy(A, K, LIN_MODE, Z*ones(nX,1)) ;
% (2) naive greedy
% [rank_set rank_set_val] = run_div_rank_naive_greedy(A, K, LIN_MODE, Z*ones(nX,1)) ;

%[rank_set rank_set_val] = run_div_rank_prune_greedy(A, K, LIN_MODE, Z*ones(nX,1)) ;

% run clustering from rank set.
[cluster_id] = run_cluster_from_rankset(A, rank_set(1:K), 1:K) ;


% % --------------------------------------------------------------%
%     Display data, selected sources and clustering
% % --------------------------------------------------------------%

colors =   [0 0 1;  1 0 0;  0 1 0;    1 0 1;  ...
            0 1 1;  1 1 0;  0 0 0;    0.78 0.25 0.25; ...
            0.5 0.5 0.5];
marksize = 20 ;
fontsize = 24 ;



% display data
fig=figure; hold on; 
sel_pnts = zeros(K,1) ;
plot(X(:,1), X(:,2),'.','Color',[0 0 1],'MarkerSize',marksize);
axis equal;
axis([-1.1 1.1 -1.1 1.1])
set(gca,'XTick',[-1:0.5:1], 'FontName','times', 'FontSize',fontsize) 
set(gca,'YTick',[-1:0.5:1], 'FontName','times', 'FontSize',fontsize) 
grid on ;
if bSave, saveas(fig,'output/div_gau_data.fig') ; end
if bSave, saveas(fig,'output/div_gau_data.png') ; end


% display rank set and clusters
fig = figure; hold on; 
sel_pnts = zeros(length(cluster_id),1) ;
for l=1:K,
    cl_ind = find(cluster_id==l) ;
    sel_pnts(cl_ind) = 1 ;
    plot(X(cl_ind,1), X(cl_ind,2),'.','Color',colors(l+1,:),'MarkerSize',marksize);
end
plot(X(sel_pnts==0,1), X(sel_pnts==0,2),'.','Color',[0 0 0],'MarkerSize',marksize);
plot(X(rank_set(1:K),1), X(rank_set(1:K),2),'o','Color',[0 0 ...
        0],'MarkerSize',marksize);
axis equal;
axis([-1.1 1.1 -1.1 1.1])
set(gca,'XTick',[-1:0.5:1], 'FontName','times', 'FontSize',fontsize) 
set(gca,'YTick',[-1:0.5:1], 'FontName','times', 'FontSize',fontsize) 
grid on ;
if bSave, saveas(fig,'output/div_gau_cluster.fig') ; end
if bSave, saveas(fig,'output/div_gau_cluster.png') ; end


% % --------------------------------------------------------------%
%     Display marginal temperature gain
% % --------------------------------------------------------------%

% compute marginal temperature gain
[obj_val rank_set_gt] = compute_obj_val(A, K+1, Z, LIN_MODE) ;

% Display marginal temperature Gain
prev_max_objval = 0 ;
for h=1:K+1,
    val_margin = obj_val(:,h) - prev_max_objval ;
    prev_max_objval = obj_val(rank_set_gt(h),h) ;
    fig =figure; plot3(X(:,1),X(:,2),val_margin,'.','MarkerSize',marksize); hold on;

    for i=1:h,
        if i==K+1, continue; end
        plot3(X(rank_set_gt(i),1), X(rank_set_gt(i),2), val_margin(rank_set_gt(i)),...
        'o','color',colors(i+1,:),'MarkerSize',marksize);
    end
    grid on; axis square;
    set(gca,'FontName','times', 'FontSize',fontsize) 
    view([60 10])
    if h==K, axis([-1 1 -1 1 0 40]), end

    if h==1,maxzaxis = 200 ;
    elseif h==2, maxzaxis = 100 ;
    else, maxzaxis = 50 ; 
    end
    axis([-1.1 1.1 -1.1 1.1 0 maxzaxis])
    if bSave, saveas(fig, ['output/div_gau_rank_set_' num2str(h) '.fig']) ; end
    if bSave, saveas(fig, ['output/div_gau_rank_set_' num2str(h) '.png']) ; end
end

end
