function [segmImg] = get_segm_img_from_sp(spImg, clusterID)
%
%   Get Cluster Img
%
% -------
% INPUTS:
%
%   spImg           : Superpixel image.
%   clusterID       : clusters of SPs
%
% --------
% OUTPUTS:
%
%   segmImg         : Output segment image.
%
% -------------
%
%   Guknhee Kim (gunhee@cs.cmu.edu)
%

if nargin<3,
    bDraw = 0 ;
end

[nr nc nch] = size(spImg) ; nch = 3 ;
numSP = max(max(spImg)) ;
numCL = max(clusterID) ;

% get segment image
spImg(spImg==0) = numSP + 1 ;
clusterID = [clusterID; 0] ;
segmImg = clusterID(spImg) ;

% fill the boundaries (Optional)
% pref_order = 1:numCL ;
[segmImg] = fill_up_boundary(segmImg) ;

% make it uint8
if max(segmImg(:))<256,
    segmImg = uint8(segmImg) ;
end

%{
% make it colored image
if nargout>1,
    cmap = fn_VOClabelcolormap(256) ;
    segmClrImg = zeros([nr, nc 3]) ;
    for i=1:3,
        segmClrImg(:,:,i) = reshape(cmap(segmImg,i),[nr nc]) ;
    end
end
%}

end

% fill the boundaries (Optional)
function [clImg] = fill_up_boundary(clImg)

    nCluster = max(max(clImg)) ;
    [nR nC] = size(clImg) ;

    while 1,
        % nonzero elements
        [nnzR nnzC] = find(clImg==0) ;
        nnzI = sub2ind([nR nC], nnzR, nnzC) ;

        % if empty, break it.
        if isempty(nnzR), break; end

        % r-1:r+1
        nnzRM = nnzR - 1 ; nnzRP = nnzR + 1 ;
        nnzRM(nnzRM<1) = 1 ;
        nnzRP(nnzRP>nR) = nR ;
        % c-1:c+1
        nnzCM = nnzC - 1 ; nnzCP = nnzC + 1 ;
        nnzCM(nnzCM<1) = 1 ;
        nnzCP(nnzCP>nC) = nC ;
        % four neighbors (r,c)-indices
        nnzRNgh = [nnzRM nnzRP nnzR nnzR] ;
        nnzCNgh = [nnzC nnzC nnzCM nnzCP] ;
        % single indices
        sz = sub2ind([nR nC], nnzRNgh, nnzCNgh) ;
        % labels
        label_cl = clImg(sz) ;

        % hist of lebels of neighbors
        hist_label_cl = histc(label_cl',0:nCluster) ;
        hist_label_cl(1,:) = [] ;

        % the labels that are the most in the neighbors
        [foo label_final] = max(hist_label_cl,[],1) ;

        % fill the labels
        clImg(nnzI) = label_final ;

        % the pixels where all neighboring pixels are 0 (ie. boundary).
        clImg(nnzI(find(sum(hist_label_cl)==0))) = 0 ;
    end

end
