function [spImg bdImg] = run_sp_ncuts(I, N_sp2, bShow)
%
%  Generate superpixels by Greg Mori
%  http://www.cs.sfu.ca/~mori/research/superpixels
%
% -------
% INPUTS:
%  
%   I :     The image to be super-pixeled.
%   numSP : The number of suepr-pixels
%   bShow : whether the result is drawn or not.
%
% --------
% OUTPUTS:
%
%   spImg :     Segment map. max(max(spImg)) = number of segments.
%   bdImg :     binary boundary map. the pixel at boundary = 1.
%
%   See the details of the algorithm in 'lib/superpixels64/sp_demo.m'.
%
% ---------------
% Gunhee Kim
%

if nargin<3,
    bShow = 0 ;
end

% add paths
lib_path = 'lib/superpixels64/';
if ~exist('segImage')
    addpath(lib_path) ;
end
if ~exist('cncut')
    addpath([lib_path 'yu_imncut']);
end
if ~exist('pbCGTG')
  addpath([lib_path 'segbench/Detectors']);
  addpath([lib_path 'segbench/Filters']);
  addpath([lib_path 'segbench/Gradients']);
  addpath([lib_path 'segbench/Textons']);
  addpath([lib_path 'segbench/Util']);
end

% I = im2double(imread('img_000070.jpg'));

N = size(I,1);
M = size(I,2);

% Number of superpixels coarse/fine.
N_sp=round(numSP/4) ;
%numSP=2*N_sp;
% Number of eigenvectors.
N_ev=40;


% ncut parameters for superpixel computation
diag_length = sqrt(N*N + M*M);
par = imncut_sp;
par.int=0;
par.pb_ic=1;
par.sig_pb_ic=0.05;
par.sig_p=ceil(diag_length/50);
par.verbose=0;
par.nb_r=ceil(diag_length/60);
par.rep = -0.005;  % stability?  or proximity?
par.sample_rate=0.2;
par.nv = N_ev;
par.sp = N_sp;

% Intervening contour using mfm-pb
fprintf('running PB\n');
[emag,ephase] = pbWrapper(I,par.pb_timing);
emag = pbThicken(emag);
par.pb_emag = emag;
par.pb_ephase = ephase;
clear emag ephase;

st=clock;
fprintf('Ncutting...');
[Sp,Seg] = imncut_sp(I,par);
fprintf(' took %.2f minutes\n',etime(clock,st)/60);

st=clock;
fprintf('Fine scale superpixel computation...');
Sp2 = clusterLocations(Sp,ceil(N*M/numSP));
fprintf(' took %.2f minutes\n',etime(clock,st)/60);

spImg = Sp2 ;
[cx,cy] = gradient(spImg);

% draw the result ?
if bShow,
    I_sp = segImage(I,Sp);
    I_sp2 = segImage(I,Sp2);
    I_seg = segImage(I,Seg);

    figure;
    subplot(1,4,1);
    imshow(I);
    subplot(1,4,2);
    imshow(I_seg);
    subplot(1,4,3);
    imshow(I_sp);
    subplot(1,4,4);
    imshow(I_sp2);
end
% save temptemp_seg.mat I_seg I_sp I_sp2 ;
