function [spImg] = get_sp_slic(I, numSP)
%
%   Generate superpixels by SLIC of VLFeat.org
%   http://www.vlfeat.org/overview/slic.html
%
% -------
% INPUTS:
%
%   I :     The image to be super-pixeled.
%   numSP : The number of suepr-pixels.
%
% --------
% OUTPUTS:
%
%   spImg :     Segment map. max(max(spImg)) = number of segments.
%
% ---------------
% Gunhee Kim
%

% default number of superpixels.
if nargin<2,
    numSP = 500 ;
end

% parameters of SLIC. See http://www.vlfeat.org/overview/slic.html
regionSize = max(round( (size(I,1)+size(I,2))/2/sqrt(numSP) ) ,1) ;
regularizer = 0.5 ;

% convert image to LAB format
% I = vl_xyz2lab(vl_rgb2xyz(I)) ;

I = run_bilateralFilt(I) ;

% run the superpixel algorithm
spImg = vl_slic(single(I), regionSize, regularizer) ;

% no zeros
spImg = spImg + 1 ;

% unique SP indices

uniqInd = unique(spImg) ;
% if some SP indices are missing, remove them.
if uniqInd(end)~=length(uniqInd)
    tmpInd(uniqInd) = 1:length(uniqInd) ;
    spImg = tmpInd(spImg) ;
end
