//
//  Make the Adjacency graph from the segment map.
//	Assume that pixels at boundaries are labelled by 0. 
//
// -------
// INPUTS:
//  (Note that there are several other parameters at the top of the code)
//
//
// --------
// OUTPUTS:
//
//
// ---------------
// Gunhee Kim
//


#include <stdio.h>
#include "mex.h"

void MakeAdjacencyList(double *L, int numLabels, int numCols, int numRows, double *AdjMax)
{
	int i, j, k, l, thisLabel ;
	int ngbLabel[8], uniqLabel[8] ;
	int pntUniqLabel = 0, bExist = 0, minL, maxL ;

	// Filling up the adjacency list
	for(i=0;i<numRows;i++)
	{
		for(j=0;j<numCols;j++)
		{
			thisLabel = (int) L[j*numRows + i];	// Label of the current pixel
		
			// If this is a boundary Pixel
			if(thisLabel==0)
			{
				// 4-neighborhood
				// N, S, W, E, NW, NE, SE, SW
				ngbLabel[0] = (i>0)            ? (int) L[j*numRows + (i-1)] : thisLabel;
				ngbLabel[1] = (i<(numRows-1))  ? (int) L[j*numRows + (i+1)] : thisLabel;
				ngbLabel[2] = (j>0)            ? (int) L[(j-1)*numRows + i] : thisLabel;
				ngbLabel[3]	= (j<(numCols-1))  ? (int) L[(j+1)*numRows + i] : thisLabel;

				// 8-neighborhood
				ngbLabel[4] = (i>0 && j>0)						? (int) L[(j-1)*numRows + (i-1)] : thisLabel;
				ngbLabel[5] = (i>0 && j<(numCols-1))			? (int) L[(j+1)*numRows + (i-1)] : thisLabel;
				ngbLabel[6] = (i<(numRows-1) && j<(numCols-1))	? (int) L[(j+1)*numRows + (i+1)] : thisLabel;
				ngbLabel[7] = (i<(numRows-1) && j>0)			? (int) L[(j-1)*numRows + (i+1)] : thisLabel;

	
				// Find unique labels in nghLable[0~7]
				pntUniqLabel = 0 ;
				for (k=0; k<8; k++) 
				{
					// If it is boundary, skip it.
					if(ngbLabel[k]==0)
						continue;
					bExist = 0 ;
					for(l=0; l<pntUniqLabel; l++)
					{
						if (uniqLabel[l]==ngbLabel[k]){
							bExist = 1 ;
							break;
						}
					}
					if (bExist==0)
					{
						uniqLabel[pntUniqLabel] = ngbLabel[k];
						++pntUniqLabel ;
					}
				}
				
				// For the unique labels,
				for (k=0; k<pntUniqLabel; k++)
				{
				  //printf("%d ",uniqLabel[k]) ;
					for(l=k+1; l<pntUniqLabel; l++)
					{
						if (uniqLabel[k]<uniqLabel[l])
						{
							minL = uniqLabel[k] ; maxL = uniqLabel[l] ;
						}
						else 
						{
							minL = uniqLabel[l] ; maxL = uniqLabel[k] ;
						}
						--minL; --maxL;
						AdjMax[minL*numLabels+maxL] = 1 ;
					}
				}
			}
		}
	}
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
	double *L, *AdjMax ;
	int numRows, numCols, numLabels, i, j ;	
	  
	if(nrhs!=2) {
		mexErrMsgTxt("Incorrect No. of inputs");
	} else if(nlhs>1) {
		mexErrMsgTxt("Too many output arguments");
	}
	  
	// boundary image (1==boundary)
	L = mxGetPr(prhs[0]);
	// number of Labels
	numLabels = (int)mxGetScalar(prhs[1]);

	numCols = mxGetN(prhs[0]);		// Image Size
	numRows = mxGetM(prhs[0]);
	
	// Memory Allocated for output array
	plhs[0] = mxCreateDoubleMatrix(numLabels, numLabels, mxREAL) ;
	// variable assigned to the output memory location
	AdjMax = (double *)mxGetPr(plhs[0]) ;

	for (i=0; i<numLabels*numLabels; i++)
	  AdjMax[i] = 0 ;

	//printf("%d %d %d\n", numLabels, numCols, numRows);
	
	// Adjacency list made
	MakeAdjacencyList(L, numLabels, numCols, numRows, AdjMax) ;
}
