//
//  Get features from each segment. 
//  (ex. Mean of RGB, HSV, L*ab, and texton)
//
// -------
// INPUTS:
//  (Note that there are several other parameters at the top of the code)
//
//
// --------
// OUTPUTS:
//
//
// ---------------
// Gunhee Kim
//


#include <stdio.h>
#include "mex.h"

#define 	max(a, b)   ((a) > (b) ? (a) : (b))
#define 	min(a, b)   ((a) < (b) ? (a) : (b))

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
	double *TspImg, *descImg, *descSP ;
	int nSeg, nPixel, dimDesc, ilabel, i, j ;	
	int *countSeg ;	

	if(nrhs!=3) {
		mexErrMsgTxt("Incorrect No. of inputs");
	} else if(nlhs>1) {
		mexErrMsgTxt("Too many output arguments");
	}

	// 1-D Segment Label Image
	TspImg = (double *)mxGetPr(prhs[0]);
	// featuve value at every pixel
	descImg = (double *)mxGetPr(prhs[1]);

	// number of Segments
	nSeg = (int)mxGetScalar(prhs[2]);
	// number of pixels
	nPixel = max(mxGetM(prhs[0]), mxGetN(prhs[0]));
	// dimension of features
	dimDesc = (int) mxGetN(prhs[1]) ;

	// Memory Allocated for output array
	plhs[0] = mxCreateDoubleMatrix(nSeg, dimDesc, mxREAL) ;

	// variable assigned to the output memory location
	descSP = (double *)mxGetPr(plhs[0]) ;

	// count of each segment
    countSeg  = (int *) mxCalloc(nSeg, sizeof(int));
	for (i=0; i<nSeg; i++) countSeg[i] = 0 ;

	//mexPrintf("%d %d %d\n", nSeg, nPixel, dimDesc) ;
	
	// for each column
	for (i=0; i<nPixel; i++)
	{
		// label at (j,i)
		ilabel = (int) TspImg[i] ;
		// if ilabel==0, it's boundary.
		if (ilabel>0) 
		{
			--ilabel ;

			// add feature values
			for (j=0; j<dimDesc; j++) 
				descSP[ilabel+j*nSeg] += descImg[i+j*nPixel] ;

			// increase count by 1.
			++countSeg[ilabel] ;
		}
	}
	
	// compute the mean
	for (i=0; i<nSeg; i++)
	{
		if (countSeg[i]==0) continue; 

		for (j=0; j<dimDesc; j++) 
			descSP[i+j*nSeg] /= countSeg[i];
	}

    mxFree(countSeg) ;
}
